{*****************************************************************************}
{**}
{**  Project Name:	DropShell}
{**     File Name:	DSAppleEvents.c}
{**}
{**   Description:	Generic AppleEvent handling routines}
{**					}
{**					This is the set of routines for handling the required Apple events.}
{**					You should NEVER have to modify this file!!!}
{**					Simply add code in DSUserProcs to the routines called by these.}
{**}
{*******************************************************************************}
{**                       A U T H O R   I D E N T I T Y}
{*******************************************************************************}
{**}
{**	Initials	Name}
{**	--------	-----------------------------------------------}
{**	LDR			Leonard Rosenthol}
{**	MTC			Marshall Clow}
{**	SCS			Stephan Somogyi}
{**   MXM        Matthew Xavier Mora}
{**}
{*******************************************************************************}
{**                      R E V I S I O N   H I S T O R Y}
{*******************************************************************************}
{**}
{**	  Date		Author	Description}
{**	---------	------	---------------------------------------------}
{** 28 Apr 95	MXM		Converted it to Metrowerks Pascal            }
{** 20 Oct 94	MXM		Converted it to Pascal                       }
{**	20 Feb 94	LDR		Modified _HandleDocs to pass item count to preflight & postflight}
{**	11 Dec 93	SCS		Universal Headers/UPPs (Phoenix 68k/PPC & PPCC)}
{**									Skipped System 6 compatible rev of DropShell source}
{**	11/24/91		LDR		Added a handler for 'pdoc' as per DTS recommendation}
{**									This caused some reorg & userProc routine changes}
{**									I also created a new common AEVT doc extractor}
{**									Cleaned up error handling by adding FailErr}
{**									Cleaned up the placement of braces}
{**									Added the passing of a userDataHandle to the odoc/pdoc routines}
{**	10/29/91		SCS		Changes for THINK C 5}
{**	10/28/91		LDR		Officially renamed DropShell (from QuickShell)}
{**									Added a bunch of comments for clarification}
{**	10/06/91		MTC		Converted to MPW C}
{**	04/09/91		LDR		Added to Projector}
{**}
{*****************************************************************************}

unit DSAppleEvents;
interface

	uses
		 DSUserProcs,DSUtils,
		
		{BJK: added these}
		AppleEvents,Events, Errors,Types,Memory,Files,ErrorsAndBeeps,globals;
		
		
		
	function HandleOAPP (var theAppleEvent: AppleEvent; var reply: AppleEvent; handlerRefcon: LongInt): OSErr;
	function HandleODOC (var theAppleEvent: AppleEvent; var reply: AppleEvent; handlerRefcon: LONGINT): OSErr;
	function HandlePDOC (var theAppleEvent: AppleEvent; var reply: AppleEvent; handlerRefcon: LONGINT): OSErr;
	function HandleQUIT (var theAppleEvent: AppleEvent; var reply: AppleEvent; handlerRefcon: LONGINT): OSErr;
	procedure DoHighLevelEvent (event: EventRecord);
	procedure InitAEVTStuff;

implementation
{}
{	This routine does all initialization for AEM, including the}
{	creation and then population of the dispatch table.}
{}
{$S Initialize}
	

	procedure InitAEVTStuff;
		var
			aevtErr: OSErr;
	begin
		aevtErr := noErr;

		if (aevtErr = noErr) then
			begin
				OAPPHandlerUPP := NewAEEventHandlerProc(@HandleOAPP);
				aevtErr := AEInstallEventHandler(kCoreEventClass, kAEOpenApplication, OAPPHandlerUPP, 0, false);
			end;


		if (aevtErr = noErr) then
			begin
				ODOCHandlerUPP := NewAEEventHandlerProc(@HandleODOC);
				aevtErr := AEInstallEventHandler(kCoreEventClass, kAEOpenDocuments, ODOCHandlerUPP, 0, false);
			end;

		if (aevtErr = noErr) then
			begin
				PDOCHandlerUPP := NewAEEventHandlerProc(@HandlePDOC);
				aevtErr := AEInstallEventHandler(kCoreEventClass, kAEPrintDocuments, PDOCHandlerUPP, 0, false);
			end;

		if (aevtErr = noErr) then
			begin
				QUITHandlerUPP := NewAEEventHandlerProc(@HandleQuit);
				aevtErr := AEInstallEventHandler(kCoreEventClass, kAEQuitApplication, QUITHandlerUPP, 0, false);
			end;

		if (aevtErr = noErr) then
			InstallOtherEvents;


		if (aevtErr <> noErr) then
			;		{ report an error if you are so included}
	end;



{	}
{	This routine is a utility routine for checking that all required }
{	parameters in the Apple event have been used.}
{}
{$S Main}
	function GotRequiredParams (var theAppleEvent: AppleEvent): OSErr;
		var
			typeCode: DescType;
			actualSize: Size;
			retErr, err: OSErr;
	begin
		err := AEGetAttributePtr(theAppleEvent, keyMissedKeywordAttr, typeWildCard, typeCode, nil, 0, actualSize);

		if (err = errAEDescNotFound) then	{ we got all the required params: all is ok}
			retErr := noErr
		else if (err = noErr) then
			retErr := errAEEventNotHandled
		else
			retErr := err;

		GotRequiredParams := retErr;
	end;

{}
{	This is another routine useful for showing debugging info.}
{	It calls the ErrorAlert routine from DSUtils to put up the }
{	error message.}
{}
{}
	procedure FailErr (err: OSErr);
	begin

		if (err <> noErr) then
		{BJK:}	Error(stringOf('[DSAppleEvents error ',err:1,']'), AppleEventNr, 0, false,false);
	end;

{	}
{	This routine is the handler for the oapp (Open Application) event.}
{	}
{	It first checks the number of parameters to make sure we got them all }
{	(even though we don't want any) and then calls the OpenApp userProc in QSUserProcs.}
{	Finally it checks to see if the caller wanted a reply & sends one, setting any error.}
{}
{$S Main}
	function HandleOAPP (var theAppleEvent: AppleEvent; var reply: AppleEvent; handlerRefcon: LongInt): OSErr;
		var
			errStr: Str255;

	begin
		FailErr(GotRequiredParams(theAppleEvent));

	{ let's show the user the splash screen}
{	ShowWindow(gSplashScreen);}

		OpenApp;		{ pass it on to the app specific routine}

		if reply.dataHandle <> nil then { a reply is sought }
			begin
				errStr := 'Opening';
				HandleOAPP := AEPutParamPtr(reply, 'errs', 'TEXT', Ptr(@errStr[1]), length(errStr));
			end
		else
			HandleOAPP := noErr;

	end;


{	}
{	This routine is the handler for the quit (Quit Application) event.}
{	}
{	It first checks the number of parameters to make sure we got them all }
{	(even though we don't want any) and then calls the QuitApp userProc in QSUserProcs.}
{	Finally it checks to see if the caller wanted a reply & sends one, setting any error.}
{}

{$S Main}
	function HandleQUIT (var theAppleEvent: AppleEvent; var reply: AppleEvent; handlerRefcon: LONGINT): OSErr;
		var
			err: OSErr;
			errStr: Str255;
	begin
		err := GotRequiredParams(theAppleEvent);
		FailErr(err);

		QuitApp;		{ pass it on to the app specific routine}

		if reply.dataHandle <> nil then { a reply is sought }
			begin
				errStr := 'Quiting';
				HandleQUIT := AEPutParamPtr(reply, 'errs', 'TEXT', Ptr(@errStr[1]), length(errStr));
			end
		else
			HandleQUIT := noErr;
	end;



{	This routine is the low level processing routine for both the }
{	odoc (Open Document) and pdoc (Print Document) events.}
{	}
{	This routine is the key one, since this is how we get the list of}
{	files/folders/disks to process.  The first thing to do is the get the}
{	list of files, and then make sure that's all the parameters (should be!).}
{	We then send call the PreflightDocs routine (from DSUserProcs), process}
{	each file in the list by calling OpenDoc (again in DSUserProcs), and finally}
{	call PostflightDocs (you know where) and setting a return value.}

{$S Main}
	function _HandleDocs (var theAppleEvent: AppleEvent; var reply: AppleEvent; opening: Boolean): OSErr;
		var
			err: OSErr;
			myFSS: FSSpec;
			docList: AEDescList;
			index, itemsInList: LongInt;
			actualSize: Size;
			keywd: AEKeyword;
			typeCode: DescType;
			userDataHandle: Handle;

	begin
		FailErr(AEGetParamDesc(theAppleEvent, keyDirectObject, typeAEList, docList));
		FailErr(GotRequiredParams(theAppleEvent));

{		How many items do we have?. }
{ NOTE: Moved here in DS 2.0 due to requests for this info in preflighter }
		err := AECountItems(docList, itemsInList);
		FailErr(err);

		if (PreFlightDocs(opening, itemsInList, userDataHandle)) then
			begin 	{ let the app do any preflighting it might need}

				for index := 1 to itemsInList do
					begin
						FailErr(AEGetNthPtr(docList, index, typeFSS, keywd, typeCode, @myFSS, sizeof(myFSS), actualSize));
						OpenDoc(@myFSS, opening, userDataHandle);	{call the open routine}
					end;

				PostFlightDocs(opening, itemsInList, userDataHandle);	{ cleanup time}
			end
		else
			err := errAEEventNotHandled;	{ tells AEM that we didn't handle it!}

		FailErr(AEDisposeDesc(docList));

		_HandleDocs := err;
	end;

{}
{	This routine is the handler for the odoc (Open Document) event.}
{	}
{	The odoc event simply calls the common _HandleDocs routines, which will}
{	do the dirty work of parsing the AEVT & calling the userProcs.}
{}
{$S Main}
	function HandleODOC (var theAppleEvent: AppleEvent; var reply: AppleEvent; handlerRefcon: LONGINT): OSErr;
	begin
		HandleODOC := _HandleDocs(theAppleEvent, reply, TRUE);	{call the low level routine}
	end; { HandleODOC }

{}
{	This routine is the handler for the pdoc (Print Document) event.}
{	}
{	The pdoc event like the odoc simply calls the common _HandleDocs routines}
{}
	function HandlePDOC (var theAppleEvent: AppleEvent; var reply: AppleEvent; handlerRefcon: LONGINT): OSErr;
	begin
		HandlePDOC := _HandleDocs(theAppleEvent, reply, FALSE);	{call the low level routine}
	end; { HandlePDOC }

{	}
{	This is the routine called by the main event loop, when a high level}
{	event is found.  Since we only deal with Apple events, and not other}
{	high level events, we just pass everything onto the AEM via AEProcessAppleEvent}
{}
	procedure DoHighLevelEvent (event: EventRecord);
	begin
		FailErr(AEProcessAppleEvent(event));
	end;
end.