{*****************************************************************************}
{**}
{**  Project Name:	DropShell}
{**     File Name:	DSUserProcs.p}
{**}
{**   Description:	Specific AppleEvent handlers used by the DropBox}
{**}
{*******************************************************************************}
{**                       A U T H O R   I D E N T I T Y}
{*******************************************************************************}
{**}
{**	Initials	Name}
{**	--------	-----------------------------------------------}
{**	LDR			Leonard Rosenthol}
{**	MTC			Marshall Clow}
{**	SCS			Stephan Somogyi}
{**	MXM			Matthew Xavier Mora}
{**}
{*******************************************************************************}
{**                      R E V I S I O N   H I S T O R Y}
{*******************************************************************************}
{**}
{**	  Date		Time	Author	Description}
{**	--------	-----	------	---------------------------------------------}
{**	06/23/94			MXM		Converted it to Pascal}
{**	06/23/94			LDR		Added support for ProcessItem and ProcessFolder handling}
{**	02/20/94			LDR		Modified Preflight & Postflight to take item count}
{**	01/25/92			LDR		Removed the use of const on the userDataHandle}
{**	12/09/91			LDR		Added the new SelectFile userProc}
{**								Added the new Install & DisposeUserGlobals procs}
{**								Modified PostFlight to only autoquit on odoc, not pdoc}
{**	11/24/91			LDR		Added the userProcs for pdoc handler}
{**								Cleaned up the placement of braces}
{**								Added the passing of a userDataHandle}
{**	10/29/91			SCS		Changes for THINK C 5}
{**	10/28/91			LDR		Officially renamed DropShell (from QuickShell)}
{**								Added a bunch of comments for clarification}
{**	10/06/91	00:02	MTC		Converted to MPW C}
{**	04/09/91	00:02	LDR		Added to Projector}
{**}
{*****************************************************************************}

unit DSUserProcs;
interface

uses
	Types, QuickDraw, Memory, StandardFile, Packages, GestaltEqu, Files, Aliases,
	 AppleEvents,ToolUtils, 
	 
	 {BJK: Added these}
		DSUtils, Events,TextUtils,Errors,globals,convert,log,ErrorsAndBeeps;



{ Static Prototypes}
	function ProcessItem (myFSSPtr: FSSpecPtr): OSErr;
	function ProcessFolder (myFSSPtr: FSSpecPtr): OSErr;
	procedure InstallOtherEvents;
	procedure OpenApp;
	procedure QuitApp;
	function PreFlightDocs (opening: Boolean; itemCount: integer; var userDataHandle: Handle): Boolean;
	procedure OpenDoc (myFSSPtr: FSSpecPtr; opening: Boolean; userDataHandle: Handle);
	procedure PostFlightDocs (opening: Boolean; itemCount: Integer; userDataHandle: Handle);
	procedure SelectFile;
	function InitUserGlobals: Boolean;
	procedure DisposeUserGlobals;

{	set qWalkFolders to true if you want each item of a dropped folder processed}
{	as an individual item; else set to false}
	const
		qWalkFolders = true;


implementation

{$S Main}
{---------------------------------------------------------------------------------------------------}
{	This routine is called during init time.}
{	}
{	It allows you to install more AEVT Handlers beyond the standard four}
{---------------------------------------------------------------------------------------------------}
	procedure InstallOtherEvents;
{---------------------------------------------------------------------------------------------------}
	begin
	end;

{$S Main}
{---------------------------------------------------------------------------------------------------}
{	This routine is called when an OAPP event is received.}
{	}
{	Currently, all it does is set the gOApped flag, so you know that}
{	you were called initally with no docs, and therefore you shouldn't }
{	quit when done processing any following odocs.}
{---------------------------------------------------------------------------------------------------}
	procedure OpenApp;
{---------------------------------------------------------------------------------------------------}
	begin
		gOApped := true;
	end;

{$S Main}
{---------------------------------------------------------------------------------------------------}
{	This routine is called when an QUIT event is received.}
{	We simply set the global done flag so that the main event loop can}
{	gracefully exit.  We DO NOT call ExitToShell for two reasons:}
{	1) It is a pretty ugly thing to do, but more importantly}
{	2) The Apple event manager will get REAL upset!}
{---------------------------------------------------------------------------------------------------}
	procedure QuitApp;
{---------------------------------------------------------------------------------------------------}
		
	begin
	
		if logWinOpen then
				Finished := closelog
			else
				Finished := true;
	end;



{$S Main}
{---------------------------------------------------------------------------------------------------}
{	This routine is the first one called when an ODOC or PDOC event is received.}
{	In this routine you would place code used to setup structures, etc. }
{	which would be used in a 'for all docs' situation (like "Archive all}
{	dropped files")}
{	}
{	Obviously, the opening boolean tells you whether you should be opening}
{	or printing these files based on the type of event recieved.}
{	}
{	NEW IN 2.0!}
{	The itemCount parameter is simply the number of items that were dropped on}
{	the application and that you will be processing.  This gives you the ability}
{	to do a single preflight for memory allocation needs, rather than doing it}
{	once for each item as in previous versions.}
{	}
{	userDataHandle is a handle that you can create & use to store your own}
{	data structs.  This dataHandle will be passed around to the other }
{	odoc/pdoc routines so that you can get at your data without using}
{	globals - just like the new StandardFile.  }
{	}
{	We also return a boolean to tell the caller if you support this type}
{	of event.  By default, our dropboxes don't support the pdoc, so when}
{	opening is FALSE, we return FALSE to let the caller send back the}
{	proper error code to the AEManager.}
{	}
{	You will probably want to remove the #pragma unused (currently there to fool the compiler!)}
{---------------------------------------------------------------------------------------------------}
	function PreFlightDocs (opening: Boolean; itemCount: integer; var userDataHandle: Handle): Boolean;
{---------------------------------------------------------------------------------------------------}
	begin
		
		if not opening then
			Error('',NoAEPrintDocNr,0,false,false);
	{ we support opening, but not printing - see above}
	PreFlightDocs := true;
	end;

{$S Main}
{---------------------------------------------------------------------------------------------------}
{	This routine is called for each file passed in the ODOC event.}
{	}
{	In this routine you would place code for processing each file/folder/disk that}
{	was dropped on top of you.}
{	}
{	You will probably want to remove the #pragma unused (currently there to fool the compiler!)}
{---------------------------------------------------------------------------------------------------}
	procedure OpenDoc (myFSSPtr: FSSpecPtr; opening: Boolean; userDataHandle: Handle);
{---------------------------------------------------------------------------------------------------}
		var
			err: OSErr;
	begin
		err := noErr;					{	For this case we need to determine if the FSSpec is a file or folder.}
													{	If it's a folder, we then need to process each item in that folder,}
													{	otherwise just process the item.}
		
		if qWalkFolders then
			begin
			if FSpIsFolder(myFSSPtr) then
					begin
					err := ProcessFolder(myFSSPtr);
					end
				else
					begin
					err := ProcessItem(myFSSPtr);
					end;
			end
		else
			begin		{For this case we just call ProcessItem on the FSSpec above.}
				err := ProcessItem(myFSSPtr);
			end
	{ you should probably do something if you get back an error ;)}
	end;


{$S Main}
{---------------------------------------------------------------------------------------------------}
{	This routine is the last routine called as part of an ODOC event.}
{	}
{	In this routine you would place code to process any structures, etc. }
{	that you setup in the PreflightDocs routine.}
{}
{	NEW IN 2.0!}
{	The itemCount parameter was the number of items that you processed.}
{	It is passed here just in case you need it ;)  }
{	}
{	If you created a userDataHandle in the PreFlightDocs routines, this is}
{	the place to dispose of it since the Shell will NOT do it for you!}
{	}
{	You will probably want to remove the #pragma unusued (currently there to fool the compiler!)}
{---------------------------------------------------------------------------------------------------}
	procedure PostFlightDocs (opening: Boolean; itemCount: Integer; userDataHandle: Handle);
{---------------------------------------------------------------------------------------------------}
	begin
		{No closing after being dropped on; DXFtoIll is not a "real" dropBox!}
		{if (opening) and (not gOApped) then
			Finished := true;	{	close everything up!}

		{	The reason we do not auto quit is based on a recommendation in the}
		{	Apple event Registry which specifically states that you should NOT}
		{	quit on a 'pdoc' as the Finder will send you a 'quit' when it is }
		{	ready for you to do so.}
	end;


{---------------------------------------------------------------------------------------------------}
{	This routine gets called for each item (which could be either a file or a folder)}
{	that the caller wants dropped.  The determining factor is the definition of the }
{	qWalkFolder compiler directive.   Either way, the item in question should be}
{	processed as a single item and not "dissected" into component units (like subfiles}
{	of a folder!)}
{---------------------------------------------------------------------------------------------------}
	function ProcessItem (myFSSPtr: FSSpecPtr): OSErr;
{---------------------------------------------------------------------------------------------------}

		var
			err: OSErr;
	begin
		
		
		
		err := noErr;

		doConvert(myFSSPtr);{in unit convert}
		ProcessItem := err;
	end;

{---------------------------------------------------------------------------------------------------}
{	This routine gets called for any folder (or disk) that the caller wants }
{	processed as a set of component items, instead of as a single entity.}
{	The determining factor is the definition of the qWalkFolder compiler directive.}
{---------------------------------------------------------------------------------------------------}
	function ProcessFolder (myFSSPtr: FSSpecPtr): OSErr;
{---------------------------------------------------------------------------------------------------}
		var
			err: OSErr;
			index, oldIndex, localIndex: Integer;
			localFSSpec, curFSSpec: FSSpec;
			cipb: CInfoPBRec;
			fName, vFName: Str255;
			dirID, origDirID: LongInt;
			foundPosition: Boolean;
	begin
		err := noErr;

 	{ copy the source locally to avoid recursion problems}
		BlockMoveData(Ptr(myFSSPtr), @localFSSpec, sizeof(FSSpec));
	{	get the dirID for THIS folder, not it's parent!}
		BlockMoveData(@localFSSpec.name, @fName, 32);
		cipb.ioCompletion := nil;
		cipb.ioNamePtr := @fName;
		cipb.ioVRefNum := localFSSpec.vRefNum;
		cipb.ioFDirIndex := 0;	{ use the dir & vRefNum;}
		cipb.ioDirID := localFSSpec.parID;
		err := PBGetCatInfoSync(@cipb);

		if (err = noErr) then
			begin
				origDirID := cipb.ioDrDirID; { copy the sucker}
				index := 1;

		{ index through all contents of this folder}
				while (err = noErr) do
					begin
						dirID := origDirID;
						localIndex := index;
						fName := '';
						cipb.ioCompletion := nil;
						cipb.ioNamePtr := @fName;
						cipb.ioVRefNum := localFSSpec.vRefNum;
						cipb.ioFDirIndex := localIndex;	{ use a real index}
						cipb.ioDirID := dirID;
						err := PBGetCatInfoSync(@cipb);

						if (err = noErr) then
							begin
								curFSSpec.vRefNum := cipb.ioVRefNum;
								curFSSpec.parID := dirID;
								curFSSpec.name := fname;
{Check to see if this entry is a folder.}
								if BAND(cipb.ioFlAttrib, ioDirMask) = ioDirMask then
									begin
										err := ProcessFolder(@curFSSpec);
									end
								else
									begin
									err := ProcessItem(@curFSSpec);
									err := err;
									end;

				{	If we've had an error, get out! }
								if (err <> noErr) then
									leave;

				{ dirID = origDirID;	}
								localIndex := index;
				{Now take into account new files being created}
				{in the current directory & messing up our index.}
				{See Dev.CD Vol. XI:Tools & Apps (Moof!):Misc Utilities:}
				{Disinfectant & Source 2.5.1:Sample:Notes:Scan Alg	}

								vFName := '';
								cipb.ioCompletion := nil;
								cipb.ioNamePtr := @vFName;
								cipb.ioVRefNum := localFSSpec.vRefNum;
								cipb.ioFDirIndex := localIndex;	{ use a real index}
								cipb.ioDirID := dirID;
								err := PBGetCatInfoSync(@cipb);
								oldIndex := index;
								if (err = noErr) then
									begin
					{	If they're equal - same place, go to next }
										if (EqualString(vFName, fName, false, false)) then
											index := index + 1;
									end;

				{	If we didn't advance, then perhaps a file was created or deleted }
								if (oldIndex = index) then
									begin
										oldIndex := index;	{ save off the old }
										index := 0;		{ and start at the beginning }
										err := noErr;
										vFName := '';
										foundPosition := false;

										while (not foundPosition) do
											begin
												index := index + 1;
												vFName := '';
												cipb.ioCompletion := nil;
												cipb.ioNamePtr := @vFName;
												cipb.ioVRefNum := localFSSpec.vRefNum;
												cipb.ioFDirIndex := index;	{ now use a real index }
												cipb.ioDirID := dirID;
												err := PBGetCatInfoSync(@cipb);

												if (err = fnfErr) then
													begin   { we've just been deleted}
														index := oldIndex;
														foundPosition := true;
														err := noErr;	{ have to remember to reset this!}
													end;

					{	found same file & same index position }
					{	so try the next item }
												if ((not foundPosition) and EqualString(fName, vFName, false, false)) then
													begin
														index := index + 1;
														foundPosition := true;
													end;
											end;
									end;
							end;
					end;
			end;

		ProcessFolder := err;
	end;

{---------------------------------------------------------------------------------------------------}
{	This routine is called when the user chooses "Select File" from the}
{	File Menu.}
{	}
{	Currently it simply calls the new StandardGetFile routine to have the}
{	user select a single file (any type, numTypes = -1) and then calls the}
{	SendODOCToSelf routine in order to process it.  }
{			}
{	The reason we send an odoc to ourselves is two fold: 1) it keeps the code}
{	cleaner as all file openings go through the same process, and 2) if events}
{	are ever recordable, the right things happen (this is called Factoring!)}
{}
{	Modification of this routine to only select certain types of files, selection}
{	of multiple files, and/or handling of folder & disk selection is left }
{	as an exercise to the reader.}
{---------------------------------------------------------------------------------------------------}
	procedure SelectFile;
{---------------------------------------------------------------------------------------------------}
		var
			stdReply: StandardFileReply;
			theTypeList: SFTypeList;
	begin
		StandardGetFile(nil, -1, @theTypeList, stdReply);
		if (stdReply.sfGood) then				{user did not cancel}
			SendODOCToSelf(stdReply.sfFile);	{so send me an event!}
	end;


{---------------------------------------------------------------------------------------------------}
{	This routine is called during the program's initialization and gives you}
{	a chance to allocate or initialize any of your own globals that your}
{	dropbox needs.}
{	}
{	You return a boolean value which determines if you were successful.}
{	Returning false will cause DropShell to exit immediately.}
{---------------------------------------------------------------------------------------------------}
	function InitUserGlobals: Boolean;
{---------------------------------------------------------------------------------------------------}
	begin
		InitUserGlobals := TRUE;	{nothing to do, it we must be successful!}
	end;


{---------------------------------------------------------------------------------------------------}
{	This routine is called during the program's cleanup and gives you}
{	a chance to deallocate any of your own globals that you allocated }
{	in the above routine.}
{---------------------------------------------------------------------------------------------------}
	procedure DisposeUserGlobals;
{---------------------------------------------------------------------------------------------------}
	begin
	{nothing to do for our sample dropbox}
	end;


end.