{*****************************************************************************}
{**}
{**  Project Name:	DropShell}
{**     File Name:	DSUtils.p}
{**}
{**   Description:	Utility routines that may be useful to DropBoxes}
{**}
{*******************************************************************************}
{**                       A U T H O R   I D E N T I T Y}
{*******************************************************************************}
{**}
{**	Initials	Name}
{**	--------	-----------------------------------------------}
{**	SCS			Stephan Somogyi		}
{**	LDR			Leonard Rosenthol	}
{**	MXM			Matthew Xavier Mora	}
{**}
{*******************************************************************************}
{**                      R E V I S I O N   H I S T O R Y}
{*******************************************************************************}
{**}
{**	  Date		Author	Description}
{**	---------	------	---------------------------------------------}
{**	20 Oct 94	MXM		Converted to Pascal}
{**	23 Jun 94	LDR		Added a bunch of new routines for Marshall}
{**	20 Feb 94	LDR		Added some new useful File System routines}
{**	11 Dec 93	SCS		Universal Headers/UPPs (Phoenix 68k/PPC & PPCC)}
{**						Skipped System 6 compatible rev of DropShell source}
{**						Changed GetAppName to GetMyAppName (StdCLib conflict)}
{**	12/09/91	LDR		Added the Apple event routines}
{**	11/24/91	LDR		Original Version}
{**}
{*****************************************************************************}

unit DSUtils;
interface
	
	uses
		Files, Dialogs, Resources, TextUtils, Quickdraw,
		Script, Processes, AppleEvents, Aliases,
		
		{BJK: Added these}
		Types,Events,OSUtils,Memory,EPPC,PPCToolbox;

type
		AEEventHandlerUPP =  ProcPtr;

		
	var
		gOApped, gHasAppleEvents, gWasEvent: Boolean;
		gEvent: EventRecord;
		OAPPHandlerUPP, ODOCHandlerUPP, PDOCHandlerUPP, QUITHandlerUPP: AEEventHandlerUPP;
		
	function FSpIsFolder (theFSSpec: FSSpecPtr): Boolean;
	procedure AddToFSSpecList (var fSpec: FSSpec; fileList: FSSpecHandle);
	procedure SendODOCToSelf (var theFileSpec: FSSpec);
	procedure SendQuitToSelf;

implementation


{** These routines use the Process Manager to give you information about yourself **}

{$SMain}
{-------------------------------------------------------------------------------}
	procedure GetMyAppName (appName: Str255);
{-------------------------------------------------------------------------------}

		var
			err: OSErr;
			info: ProcessInfoRec;
			curPSN: ProcessSerialNumber;
	begin
		err := GetCurrentProcess(curPSN);

		info.processInfoLength := sizeof(ProcessInfoRec);	{ ALWAYS USE sizeof!}
		info.processName := @appName;							{ so it returned somewhere}
		info.processAppSpec := nil;							{ I don't care!}

		err := GetProcessInformation(curPSN, info);
	end;

{$S Main}
{-------------------------------------------------------------------------------}
	procedure GetAppFSSpec (var appSpec: FSSpec);
{-------------------------------------------------------------------------------}
		var
			err: OSErr;
			appName: Str255;
			info: ProcessInfoRec;
			curPSN: ProcessSerialNumber;
	begin
		err := GetCurrentProcess(curPSN);

		info.processInfoLength := sizeof(ProcessInfoRec);	{ ALWAYS USE sizeof!}
		info.processName := @appName;							{ so it returned somewhere}
		info.processAppSpec := @appSpec;						{ so it can get returned!}

		err := GetProcessInformation(curPSN, info);
	end;

{  File Routines begin here  }

{	This routine is used to force the Finder (as much as is possible) to update}
{	information about a newly changed file or folder.  }
{	It does this by changing the modification date of the surrounding folder.}

{-------------------------------------------------------------------------------}
	function ForceFinderUpdate (var pFSS: FSSpec; flush: Boolean): OSErr;
{-------------------------------------------------------------------------------}
		var
			lErr: OSErr;
			lCBlk: CInfoPBRec;
	begin
		if (pFSS.parID <> 1) then							{ if it's a vol then reuse the NameStr}
			lCBlk.ioNamePtr := nil;

		lCBlk.ioVRefNum := pFSS.vRefNum;
		lCBlk.ioDrDirID := pFSS.parID;
		lCBlk.ioFDirIndex := 0;
		lCBlk.ioCompletion := nil;

		lErr := PBGetCatInfoSync(@lCBlk);
		if lErr = noErr then
			begin
				GetDateTime(lCBlk.ioDrMdDat);
				lCBlk.ioDrDirID := pFSS.parID;
				lErr := PBSetCatInfoSync(@lCBlk);

				if (lErr = noErr) and (flush) then
					lErr := FlushVol(nil, pFSS.vRefNum);
			end;

		ForceFinderUpdate := lErr;
	end;

{	Is the file in use (busy)? }
{-------------------------------------------------------------------------------}
	function FSpIsBusy (theFile: FSSpecPtr): Boolean;
{-------------------------------------------------------------------------------}

		var
			isBusy: Boolean;
			err: OSErr;
			cipb: CInfoPBRec;
	begin
		isBusy := false;
		cipb.ioCompletion := nil;

		cipb.ioNamePtr := @theFile^.name;
		cipb.ioVRefNum := theFile^.vRefNum;
		cipb.ioFDirIndex := 0;
		cipb.ioDirID := theFile^.parID;
		err := PBGetCatInfoSync(@cipb);
		if err = noErr then
			begin
				isBusy := (BTST(longint(cipb.ioFlAttrib), 7));	{ bit 7 = either fork open}
			end;
		FSpIsBusy := isBusy;
	end;

{	Is the file represented by this FSSpec really a folder? }
{-------------------------------------------------------------------------------}
	function FSpIsFolder (theFSSpec: FSSpecPtr): Boolean;
{-------------------------------------------------------------------------------}
		var
			err: OSErr;
			pb: CInfoPBRec;
			fName: Str255;
	begin
		if (theFSSpec^.parID = fsRtParID) then	{ it's a volume!!}
			FSpIsFolder := true;

		BlockMove(@theFSSpec^.name, @fName, 32);
		with pb do
			begin
				ioDirID := theFSSpec^.parID;
				ioCompletion := nil;
				ioNamePtr := @fName;
				ioVRefNum := theFSSpec^.vRefNum;
				ioFDirIndex := 0;
				ioFVersNum := 0;
			end;
		err := PBGetCatInfoSync(@pb);

		if (err = noErr) then
			begin
				{if BTST(longint(pb.ioFlAttrib), ioDirMask) then}
				if BAND(pb.ioFlAttrib, ioDirMask) = ioDirMask then
					FSpIsFolder := (true)
				else
					FSpIsFolder := (false);
			end;
		

	end;

{ Creates an empty handle to which FSSpecs could be added later!}
{-------------------------------------------------------------------------------}
	function NewFSSpecList: FSSpecHandle;
{-------------------------------------------------------------------------------}
	begin
		NewFSSpecList := FSSpecHandle(NewHandle(0));
	end;

{ Releases memory used by an FSSpecList when all done}
{-------------------------------------------------------------------------------}
	procedure DisposeFSSpecList (fsList: FSSpecHandle);
{-------------------------------------------------------------------------------}
	begin
		DisposeHandle(Handle(fsList));
	end;

{ Adds the new FSSpec to the end of an FSSpecList}
{-------------------------------------------------------------------------------}
	procedure AddToFSSpecList (var fSpec: FSSpec; fileList: FSSpecHandle);
{-------------------------------------------------------------------------------}
		var
{curSize, newSize: Size;}
{numFiles: LongInt;}
			err: Integer;
	begin
		if (fileList = nil) then
			exit(AddToFSSpecList);	{ if either is bogus, get out fast}

{    curSize := GetHandleSize(Handle(fileList));}
{    numFiles := curSize div sizeof(FSSpec);}
{    newSize := curSize + sizeof(FSSpec);}
{    SetHandleSize(Handle(fileList), newSize);}
{    if (MemError <> 0) then}
{    exit(AddToFSSpecList);{ any problems, get out}

{    dest := ptr(ord(fileList^) + (longint(numFiles) * sizeof(FSSpec)));}
{    BlockMove(@fSpec, dest, sizeof(FSSpec));}

		err := PtrAndHand(@fSpec, Handle(fileList), sizeof(FSSpec));

	end;


{  Apple event routines begin here  }

{}
{	This routine will create a targetDesc for sending to self.}
{}
{	We take IM VI's advice and use the typePSN form with }
{	kCurrentProcess as the targetPSN.}
{}
{-------------------------------------------------------------------------------}
	function GetTargetFromSelf (var targetDesc: AEAddressDesc): OSErr;
{-------------------------------------------------------------------------------}
		var
			psn: ProcessSerialNumber;

	begin
		psn.highLongOfPSN := 0;
		psn.lowLongOfPSN := kCurrentProcess;

		GetTargetFromSelf := AECreateDesc(typeProcessSerialNumber, @psn, sizeof(ProcessSerialNumber), targetDesc);

	end;

{ This routine will create a targetDesc using the apps signature }
{-------------------------------------------------------------------------------}
	function GetTargetFromSignature (processSig: OSType; var targetDesc: AEAddressDesc): OSErr;
{-------------------------------------------------------------------------------}
	begin
		GetTargetFromSignature := AECreateDesc(typeApplSignature, @processSig, sizeof(processSig), targetDesc);
	end;

{ This routine will create a targetDesc by bringing up the PPCBrowser }
{-------------------------------------------------------------------------------}
	function GetTargetFromBrowser (promptStr: Str255; var targetDesc: AEAddressDesc): OSErr;
{-------------------------------------------------------------------------------}
		var
			err: OSErr;
			theTarget: TargetID;
			portInfo: PortInfoRec;
	begin
		err := PPCBrowser(promptStr, '', false, theTarget.location, portInfo, nil, '');
		if (err = noErr) then
			begin
				theTarget.name := portInfo.name;
				err := AECreateDesc(typeTargetID, @theTarget, sizeof(TargetID), targetDesc);
			end;
		GetTargetFromBrowser := err;

	end;



{	This routine is the low level routine used by the SendODOCToSelf}
{	routine.  It gets passed the list of files (in an AEDescList)}
{	to be sent as the data for the 'odoc', builds up the event}
{	and sends off the event.  }
{}
{	It is broken out from SendODOCToSelf so that a SendODOCListToSelf could}
{	easily be written and it could then call this routine - but that is left}
{	as an exercise to the reader.}
{	}
{	Read the comments in the code for the order and details}

{-------------------------------------------------------------------------------}
	procedure _SendDocsToSelf (var aliasList: AEDescList);
{-------------------------------------------------------------------------------}
		var
			err: OSErr;
			theTarget: AEAddressDesc;
			openDocAE, replyAE: AppleEvent;
	begin
{}
{	First we create the target for the event.   We call another}
{	utility routine for creating the target.}
{}
		err := GetTargetFromSelf(theTarget);
		if (err = noErr) then
			begin
		{ Next we create the Apple event that will later get sent. }
				err := AECreateAppleEvent(kCoreEventClass, kAEOpenDocuments, theTarget, kAutoGenerateReturnID, kAnyTransactionID, openDocAE);

				if (err = noErr) then
					begin
			{ Now add the aliasDescList to the openDocAE }
						err := AEPutParamDesc(openDocAE, keyDirectObject, aliasList);

						if (err = noErr) then
							begin
				{}
{					and finally send the event}
{					Since we are sending to ourselves, no need for reply.}
{				}
								err := AESend(openDocAE, replyAE, kAENoReply + kAECanInteract, kAENormalPriority, 3600, nil, nil);

				{}
{					NOTE: Since we are not requesting a reply, we do not need to}
{					need to dispose of the replyAE.  It is there simply as a }
{					placeholder.}
{				}
							end;

		{	}
{			Dispose of the aliasList descriptor}
{			We do this instead of the caller since it needs to be done}
{			before disposing the AEVT}
{		}
						err := AEDisposeDesc(aliasList);
					end;

	{and of course dispose of the openDoc AEVT itself}
				err := AEDisposeDesc(openDocAE);
			end;
	end;

{}
{	This is the routine called by SelectFile to send a single odoc to ourselves.}
{	}
{	It calls the above low level routine to do the dirty work of sending the AEVT -}
{	all we do here is build a AEDescList of the file to be opened.}
{}
{-------------------------------------------------------------------------------------------------}
	procedure SendODOCToSelf (var theFileSpec: FSSpec);
{-------------------------------------------------------------------------------------------------}
		var
			err: OSErr;
			aliasList: AEDescList;
			aliasDesc: AEDesc;
			aliasH: AliasHandle;
	begin
	{Create the descList to hold the list of files}
		err := AECreateList(nil, 0, false, aliasList);

		if (err = noErr) then
			begin
		{ First we setup the type of descriptor }
				aliasDesc.descriptorType := typeAlias;

		{}
{			Now we add the file to descList by creating an alias and then}
{			adding it into the descList using AEPutDesc}
{		}
				err := NewAlias(nil, theFileSpec, aliasH);
				aliasDesc.dataHandle := Handle(aliasH);
				err := AEPutDesc(aliasList, 0, aliasDesc);
				DisposeHandle(Handle(aliasH));

		{Now call the real gut level routine to do the dirty work}
				_SendDocsToSelf(aliasList);

		{_SendDocsToSelf will dispose of aliasList for me}
			end;
	end;


{-------------------------------------------------------------------------------}
	procedure SendQuitToSelf;
{-------------------------------------------------------------------------------}
		var
			err: OSErr;
			theTarget: AEAddressDesc;
			quitAE, replyAE: AppleEvent;
	begin
		{First we create the target for the event.   We call another}
		{utility routine for creating the target.}
		err := GetTargetFromSelf(theTarget);
		if (err = noErr) then
			begin
			{ Next we create the Apple event that will later get sent. }
				err := AECreateAppleEvent(kCoreEventClass, kAEQuitApplication, theTarget, kAutoGenerateReturnID, kAnyTransactionID, quitAE);

				if (err = noErr) then
					begin

				{and finally send the event}
				{Since we are sending to ourselves, no need for reply.}
						err := AESend(quitAE, replyAE, kAENoReply + kAECanInteract, kAENormalPriority, 3600, nil, nil);
				{NOTE: Since we are not requesting a reply, we do not need to}
				{need to dispose of the replyAE.  It is there simply as a }
				{placeholder.}
					end;

				{ and of course dispose of the quit AEVT itself }
				err := AEDisposeDesc(quitAE);
			end;
	end;
end.