unit bezier;


{ ############  DXF TO ILLUSTRATOR #############################}
{    1991-1994 Universiteit Utrecht, FRW, Vakgroep Kartografie &                            }
{                           International Institute for Aerospace Survey And Earth Sciences ITC  }
{ 	Version 1.1.2  -  Main Parts Programmed by J.C. Blankert , 1991
		Version 2.2OS - code and app made OpenSource July 2002 BJK                            }
{ ########################################################}

interface
	uses
		{Macintosh Toolbox interfaces}
	fp, Printing, globals,myWindows;
	
	const
		maxsegments = 100;
	type
		pointtype = record
				x, y: integer;
			end;
		polygontype = array[0..100] of pointtype;

	procedure bezier (var p1, p2, p3, p4: punttype; segments: integer; var polygon: polygontype);
	procedure BulgeToBezierPoints (p1, p4: punttype; bulge: real; var p2, p3: punttype);

implementation

	var
		smoothval: integer;
		blendingval: array[0..maxsegments, 0..3] of real;

	procedure bezier;
		var
			j: integer;
		procedure calculate;
			var
				j, k, l: integer;
				u, b: real;
		begin
			smoothval := segments - 1;
			if segments > maxsegments then
				begin
{'Too many segments in procedure Bezier !'}
					exit(bezier);
				end;
			for j := 0 to smoothval do
				begin
					u := j / smoothval;
					for k := 0 to 3 do
						begin
							case k of
								0: 
									b := 1;
								1: 
									b := 3;
								2: 
									b := 3;
								3: 
									b := 1;
							end;
							for l := 1 to k do
								b := b * u;
							for l := 1 to 3 - k do
								b := b * (1 - u);
							blendingval[j, k] := b;
						end;
				end;
		end;
	begin
		if segments <> smoothval + 1 then
			calculate;
		for j := 0 to smoothval do
			begin
				polygon[j].x := roundtol(p1.x * blendingval[j, 0] + p2.x * blendingval[j, 1] + p3.x * blendingval[j, 2] + p4.x * blendingval[j, 3]);
				polygon[j].y := roundtol(p1.y * blendingval[j, 0] + p2.y * blendingval[j, 1] + p3.y * blendingval[j, 2] + p4.y * blendingval[j, 3]);
			end;
	end;

	procedure BulgeToBezierPoints (p1, p4: punttype; bulge: real; var p2, p3: punttype);
		var
			le, he: real;
	begin
		bulge := -bulge;
		he := 2 * bulge / 3;
		le := he / tan(2 * arctan(bulge));
		p2.x := p1.x + ((p4.x - p1.x) * le - (p4.y - p1.y) * he);
		p2.y := p1.y + ((p4.y - p1.y) * le + (p4.x - p1.x) * he);
		p3.x := p4.x + ((p1.x - p4.x) * le + (p1.y - p4.y) * he);
		p3.y := p4.y + ((p1.y - p4.y) * le - (p1.x - p4.x) * he);
	end;
end.