unit collect;


{ ############  DXF TO ILLUSTRATOR #############################}
{    1991-1994 Universiteit Utrecht, FRW, Vakgroep Kartografie &                            }
{                           International Institute for Aerospace Survey And Earth Sciences ITC  }
{ 	Version 1.1.2  -  Main Parts Programmed by J.C. Blankert , 1991                            }
{	Version 2.1 	- Programmed by B.J. Kbben:																}
{						- added Out_begin_block and Out_end_block - 8/94							
		Version 2.2OS - code and app made OpenSource July 2002 BJK	}
{ ########################################################}


interface
	uses
		{UIP}
	QuickDraw,memory,windows,fp,QuickDrawText,fonts,events,
	Dialogs,Types,TextEdit,TextUtils,Processes,files,standardfile,
	{MyUnits}
			Ctm, globals, ErrorsAndBeeps, scale, bezier,layers;
	

	procedure Out_Line (sx, sy, ex, ey: real);
	procedure Out_InitPolyLine (x, y, startwidth: real);
	procedure Out_PolyLine (x, y, bulge: real);
	procedure Out_ClosePolyLine (bulge: real; closed, fill: boolean);
	procedure Out_Pixel (x, y: real);
	procedure Out_Circle (x, y, rad: real);
	procedure Out_Arc (x, y, rad, stangle, endangle: real);
	procedure Out_Text (var theText: textRecord);
	procedure Out_Begin_Block;
	procedure Out_End_Block;

implementation

	var
		p1, FirstPoint: punttype;
		polygon: polygontype;
		a, b, c, nullpoint: punttype;
		tempRgn: RgnHandle;
		LWChanged: boolean;



	{-------------------------------}
	procedure Out_Begin_Block;
	begin
		AddElem('u',nullpoint, nullpoint, nullpoint);
	end;

	{-------------------------------}
	procedure Out_End_Block;
	begin
		AddElem('U', nullpoint, nullpoint, nullpoint);
	end;

	{-------------------------------}
	procedure Out_Line (sx, sy, ex, ey: real);
	begin
		a.x := sx;
		a.y := sy;
		AddElem('m', a, nullpoint, nullpoint);
		b.x := ex;
		b.y := ey;
		AddElem('l', b, nullpoint, nullpoint);
		AddElem('S', nullpoint, nullpoint, nullpoint);
		if not doBackground then
			begin 
			if itsABlock then foreColor(blueColor);
			TransForm(ScrCtm, sx, sy);
			TransForm(ScrCtm, ex, ey);
			Moveto(roundtol(sx), roundtol(sy));
			lineto(roundtol(ex), roundtol(ey));
			foreColor(blackColor);
			end;
		
	end;

	{-------------------------------}
	procedure drawpolygon (numpoints: integer; var polypts: polygontype);
		var
			tel: integer;
	begin
		if not doBackground then
			begin 
			if itsABlock then foreColor(blueColor);
			for tel := 0 to numpoints - 1 do
				lineto(polypts[tel].x, polypts[tel].y);
			foreColor(blackColor);
			end;
		
	end;

	{-------------------------------}
	procedure getcenter (sp, ep: punttype; bulge: real; var center: punttype);
		var
			factor, angle: real;
	begin
		angle := 2 * arctan(bulge);
		factor := (angle / pi);
		if abs((factor - trunc(factor)) - 0.5) < 0.01 then
			begin
				center.y := (sp.y + ep.y) / 2;
				center.x := (sp.x + ep.x) / 2;
			end
		else
			begin
				factor := 2 * tan(angle);
				center.y := (sp.y + ep.y) / 2 - (sp.x - ep.x) / factor;
				center.x := (sp.x + ep.x) / 2 + (sp.y - ep.y) / factor;
			end;
	end;

	{-------------------------------}
	procedure drawbulge (x, y, bulge: real);
		var
			LastPoint: punttype;
			p2, p3, p4, hulp, hulp2, center: punttype;

		function sign (getal: real): integer;
		begin
			if getal < 0 then
				sign := -1
			else if getal > 0 then
				sign := 1
			else
				sign := 0;
		end;

	begin
		p4.x := x;
		p4.y := y;
		LastPoint := p4;
		if abs(bulge) > 0.4142136 then
			begin
				getcenter(p1, p4, bulge, center);
				while abs(bulge) > 0.4142136 do
					begin
						hulp.x := center.x + (center.y - p1.y);
						hulp.y := center.y - (center.x - p1.x);
						BulgeToBezierPoints(p1, hulp, 0.4142136 * sign(bulge), p2, p3);
						AddElem('c', p2, p3, hulp);
						hulp2 := hulp;
						transform(scrctm, p1.x, p1.y);
						transform(scrctm, p2.x, p2.y);
						transform(scrctm, p3.x, p3.y);
						transform(scrctm, hulp.x, hulp.y);
						bezier(p1, p2, p3, hulp, 15, polygon);
						drawpolygon(15, polygon);
						bulge := tan(arctan(bulge) - sign(bulge) * pi / 8);
						p1 := hulp2;
					end;
			end;
		BulgeToBezierPoints(p1, p4, bulge, p2, p3);
		AddElem('c', p2, p3, p4);
		transform(scrctm, p1.x, p1.y);
		transform(scrctm, p2.x, p2.y);
		transform(scrctm, p3.x, p3.y);
		transform(scrctm, p4.x, p4.y);
		bezier(p1, p2, p3, p4, 15, polygon);
		drawpolygon(15, polygon);
		p1 := LastPoint;
	end;

	{-------------------------------}
	procedure Out_InitPolyLine (x, y, startwidth: real);
	begin
		if itsABlock then foreColor(blueColor);
		LWChanged := false;
		if (startwidth <> 0) and (SetUp^^.Lineweightsubstitution) then
			begin
				p1.x := startwidth;
				p1.y := 0;
				AddElem('w', p1, nullpoint, nullpoint);
				if not itsaBlock then {keep blocks blue}
					foreColor(greenColor);
				LWchanged := true;
			end;
		p1.x := x;
		p1.y := y;
		AddElem('m', p1, nullpoint, nullpoint);
		FirstPoint := p1;
		if not doBackground then
			begin 
			transform(ScrCTM, x, y);
			moveto(roundtol(x), roundtol(y));
			end;
		
		ForeColor(blackColor);
	end;

	{-------------------------------}
	procedure Out_PolyLine (x, y, bulge: real);
	begin
		if itsABlock then foreColor(blueColor);
		if (LWchanged) and not itsaBlock then
			foreColor(greenColor);
		if bulge <> 0 then
			drawbulge(x, y, bulge)
		else
			begin
				p1.x := x;
				p1.y := y;
				AddElem('l', p1, nullpoint, nullpoint);
				if not doBackground then
					begin 
					transform(ScrCtm, x, y);
					lineto(roundtol(x), roundtol(y));
					end;
				
			end;
		ForeColor(blackColor);
	end;

	{-------------------------------}
	procedure Out_ClosePolyLine (bulge: real; closed, fill: boolean);
		var
			x, y: real;
	begin
		if itsABlock then foreColor(blueColor);
		if FirstPoint.x = p1.x then
			if FirstPoint.y = p1.y then {closed arc or polyline}
				begin
					if fill then
						AddElem('b', nullpoint, nullpoint, nullpoint)
					else
						AddElem('s', nullpoint, nullpoint, nullpoint);
					if  (LWchanged) then {linewidth was changed, reset}
						begin
							AddElem('W', nullpoint, nullpoint, nullpoint); {no valid Illustrator => just to reset lineWidth}
							LWchanged := false;
							if not itsaBlock then{keep blocks blue}
								foreColor(blackColor);
						end;
					exit(Out_ClosePolyLine);
				end;
		if not closed then
			begin
				AddElem('S', nullpoint, nullpoint, nullpoint);
				if (SetUp^^.Lineweightsubstitution) and (LWchanged) then {linewidth was changed, reset}
					begin
						AddElem('W', nullpoint, nullpoint, nullpoint);
						LWchanged := false;
						if not itsaBlock then{keep blocks blue}
							foreColor(blackColor);
					end;
				exit(Out_ClosePolyLine);
			end;
		if bulge <> 0 then
			begin
				drawbulge(FirstPoint.x, FirstPoint.y, bulge);
			end
		else
			begin
				x := FirstPoint.x;
				y := FirstPoint.y;
				AddElem('l', FirstPoint, nullpoint, nullpoint);
				if not doBackground then
					begin 
					transform(ScrCtm, x, y);
					lineto(roundtol(x), roundtol(y));
					end;
			end;
		p1 := FirstPoint;
		if fill then
			AddElem('b', nullpoint, nullpoint, nullpoint)
		else
			AddElem('s', p1, nullpoint, nullpoint);
		if (SetUp^^.Lineweightsubstitution) and (LWchanged) then {linewidth was changed, reset}
			begin
				AddElem('W', nullpoint, nullpoint, nullpoint);
				LWchanged := false;
			end;
		foreColor(blackColor);
	end;

	{-------------------------------}
	procedure Out_Pixel (x, y: real);
		var
			a: punttype;
	begin
		a.x := x;
		a.y := y;
		AddElem('m', a, nullpoint, nullpoint);
		AddElem('l', a, nullpoint, nullpoint);
		AddElem('S', nullpoint, nullpoint, nullpoint);
		if not doBackground then
			begin 
			transform(ScrCtm, x, y);
			moveto(roundtol(x), roundtol(y));
			lineto(roundtol(x), roundtol(y));
			end;
	end;

	{-------------------------------}
	procedure Out_Circle (x, y, rad: real);
	begin
		Out_InitPolyLine(x + rad, y, 0);
		Out_PolyLine(x - rad, y, 1);
		if setUp^^.FillCircle then
			Out_ClosePolyLine(1, true, true)
		else
			Out_ClosePolyLine(1, true, false);
	end;

	{-------------------------------}
	procedure Out_Arc (x, y, rad, stangle, endangle: real);
	begin
		if stangle < 0 then
			stangle := stangle + 2 * pi;
		if endangle < 0 then
			endangle := endangle + 2 * pi;
		if endangle < stangle then
			endangle := endangle + 2 * pi;
		Out_InitPolyLine(x + rad * cos(stangle), y + rad * sin(stangle), 0);
		Out_PolyLine(x + rad * cos(endangle), y + rad * sin(endangle), abs(tan((stangle - endangle) / 4)));
		Out_ClosePolyLine(0, False, true);
	end;

	{-------------------------------}
	procedure Out_Text (var theText: TextRecord);
	begin
		AddTekst(theText);
	end;


end. {of unit collect}