unit convert;

{ ############  DXF TO ILLUSTRATOR #############################}
{    1991-1994 Universiteit Utrecht, FRW, Vakgroep Kartografie &                            }
{                           International Institute for Aerospace Survey And Earth Sciences ITC  }
{ 	Version 1.1.2  -  Main Parts Programmed by J.C. Blankert , 1991                            }
{ 	Version 2.0     -  Changes programmed by B.J. Kbben, november/december 1992      }
{ 	 							- Scaling: IllMaxX and Y are now reals instead of constants            }
{ 	 											These are set in the new unit Scale					            }
{	Version 2.1		- Programmed by B.J. Kbben															}
{									- Message logging cleaned up & logging always on, 3/94		
		Version 2.2OS - code and app made OpenSource July 2002 BJK	}
{ ########################################################}

interface
	uses
		{UIP}
	QuickDraw,memory,windows,fp,QuickDrawText,fonts,menus,processes,
	Dialogs,Types,TextEdit,TextUtils,toolutils,OSutils,files,
		{MyUnits}
		globals, ErrorsAndBeeps, log, Scale, myWindows,  myFIles, tables, 
		Setup, Layers, Entities, Sections;

		

	procedure doConvert(droppedFileSpecs: FSSpecPtr);
	procedure ConvertDxf_File(droppedFileSpecs: FSSpecPtr);

implementation

{-----------------------------------------------------------}
	procedure WriteResultsToLogFile;
		var
			message: str255;
			
	begin
		if HatchAsked then
			begin
				writeLog(stringOf(stringlist(ContainsNr), stringList(hatchNr), ':'));
				message := concat('   ', stringList(hatchNr));
				if not AskHatch then
					message := concat(message, ' ', stringList(notNr));
				message := concat(message, ' ', stringList(IncludedInNr));
				writeLog(message);
			end;
		if CurveAsked then
			begin
				writeLog(stringOf(stringlist(ContainsNr), stringList(CurveFitNr), ':'));
				message := concat('   ', stringList(CurveFitNr));
				if not Ask_Curve_Fit then
					message := concat(message, ' ', stringList(notNr));
				message := concat(message, ' ', stringList(IncludedInNr));
				writeLog(message);
			end;
		if SplineAsked then
			begin
				writeLog(stringOf(stringlist(ContainsNr), stringList(SplineFitNr), ':'));
				message := concat('   ', stringList(SplineFitNr));
				if not Ask_Spline_Fit then
					message := concat(message, ' ', stringList(notNr));
				message := concat(message, ' ', stringList(IncludedInNr));
				writeLog(message);
			end;
		if Line3DAsked then
			begin
				writeLog(stringOf(stringlist(ContainsNr), stringList(threeDLineNr), ':'));
				message := concat('   ', stringList(threeDLineNr));
				if not Ask_3D_Line then
					message := concat(message, ' ', stringList(notNr));
				message := concat(message, ' ', stringList(IncludedInNr));
				writeLog(message);
			end;
		if SetUp^^.scaling = ToCustom then {custom scaling}
			begin
				CustomX := SetUp^^.customScale;
				Message := concat(stringlist(ScaleToNr + byte(SetUp^^.scaling)), StringOf((CustomX * eenheid) : 1 : 7, ' ', EenheidStr));
			end
		else
			Message := concat(stringlist(ScaleToNr + byte(SetUp^^.scaling)), StringOf((illscale / points) : 1 : 7, ' mm.'));
		writelog(message);
		
		writeLog(stringof('DXF ', stringList(FileSizeNr), (FileSize(DxfFileNo) / 1024) : 1 : 1, ' Kbyte.'));
		writelog(concat(stringlist(DXFversNr),ACADVersion));
		writeLog(stringof('Illustrator ', stringList(FileSizeNr), (FileSize(IllFileNo) / 1024) : 1 : 1, ' Kbyte.'));
		writeLog(concat(stringList(convCompleteNr), '(',DateTimeStr(2), ')'));
	end; {of procedure}


{-----------------------------------------------------------}
	procedure ConvertDxf_File(droppedFileSpecs: FSSpecPtr);
		var
			ThisFileSize: longint;
			i: byte;
			temp: str255;
	begin
		aborted := false;
		numElems := 0;
		if openfiles(droppedFileSpecs) then
			begin
				writeLog(' ');
				writeLog(StringOf(stringlist(ConvStartedNr), ' (',  DateTimeStr(0), ')'));
				writeLog(StringOf(stringlist(InputFileNr), GetFname(DxfFileNo)));
				writeLog(StringOf(stringlist(OutputFileNr), GetFname(IllFileNo)));
				if thisISDemo then
					begin
					ThisFileSize := roundtol(FileSize(DxfFileNo) / 1024) + 1;
					if ThisFileSize > 275 then
						begin
						Error(stringlist(ToBigForDemoNr), 0, 0, true, false);
						closeallfiles;
						exitToShell;
						end;
					end;

				if (SetUp^^.TargetAppl = Ill11) then
					SetSetUpMessage(TargetApplNr, stringList(OtherApplNr))
				else if (SetUp^^.TargetAppl = Freehand) then
					SetSetUpMessage(TargetApplNr, stringList(FreehandFormatNr))
				else if (SetUp^^.TargetAppl = Ill50) then
					SetSetUpMessage(TargetApplNr, stringList(layerFormatNr));

				case SetUp^^.Grouping of
					full: 
						SetSetUpMessage(FullGroupNr, '');
					byLayer: 
						SetSetUpMessage(LayerGroupNr, '');
					none: 
						SetSetUpMessage(NoGroupNr, '');
				end;{case}

				if SetUp^^.Tielines then
					begin
						SetSetUpMessage(TieLinesYesNr, '');
					end;

				if SetUp^^.LineweightSubstitution then
					begin
						SetSetUpMessage(LSOnNr, '');
					end
				else
					SetSetUpMessage(LSoffNr, '');

				if SetUp^^.FontSubstitution then
					begin
						SetSetUpMessage(FSOnNr, '');
						for i := 1 to MaxNumFonts do
							if length(Setup^^.FontTable[i]) > 1 then
								begin
									temp := concat('   ', copy(Setup^^.FontTable[i], 1, pos('\', Setup^^.FontTable[i]) - 1), ' -> ', copy(Setup^^.FontTable[i], pos('\', Setup^^.FontTable[i]) + 1, length(Setup^^.FontTable[i])));
									SetSetUpMessage(0, temp);
								end;
					end
				else
					SetSetUpMessage(FSoffNr, '');

				if SetUp^^.ColorSubstitution then
					begin
						SetSetUpMessage(CSOnNr, '');
						for i := 1 to MaxNumFonts do
							if length(Setup^^.ColorTable[i]) > 1 then
								begin
									temp := concat('   ', copy(Setup^^.ColorTable[i], 1, pos('\', Setup^^.ColorTable[i]) - 1), ' -> ', copy(Setup^^.ColorTable[i], pos('\', Setup^^.ColorTable[i]) + 1, length(Setup^^.ColorTable[i])));
									SetSetUpMessage(0, temp);
								end;
					end
				else
					SetSetUpMessage(CSoffNr, '');

				if SetUp^^.FillSolid then
					SetSetUpMessage(FillSolidNr, '');
				if SetUp^^.FillCircle then
					SetSetUpMessage(FillCircleNr, '');
				if SetUp^^.FillPoly then
					SetSetUpMessage(FillPolyNr, '');

				if SetUp^^.VerboseLogging then
					SetSetUpMessage(VerboseLoggingNr, '');
					
				if SetUp^^.groupBlocks then
					SetSetUpMessage(groupBlocksNr, '')
				else
					SetSetUpMessage(NotNr,stringlist(groupBlocksNr));
					
				if SetUp^^.ConverterStamp then
					SetSetUpMessage(ConversionStampNr, '');

				if not DrwWinInited then
					CreateDrawingWindow(concat(stringList(Generalnr),'',GetFName(DXFFileno),''));
				InitBalk(0,FileSize(DxfFileNo));

				if readsections then
					;
					
												
				WriteResultsToLogFile;
				CloseFile(IllFileNo);
				CloseFile(DxfFileNo);
			end;
		
		{cleaning up in case of abort oid}
		if DrwWinInited then RemoveDrawingWindow;
		closeAllFiles; 
		DXFLineNR := 0;
		numElems := 0;
		
		
	end;
	
{-----------------------------------------------------------}
	procedure doConvert(droppedFileSpecs: FSSpecPtr);
	
	begin
		
		ThreedsolidWarned:=false;ThreedfaceWarned:=false; attdefWarned:=false; attribWarned:=false;
		bodyWarned:=false; dimensionWarned:=false; ellipseWarned:=false; leaderWarned:=false; 
		mlineWarned:=false; mtextWarned:=false; oleframeWarned:=false; rayWarned:=false; 
		regionWarned:=false; shapeWarned:=false; splineWarned:=false; toleranceWarned:=false;
		traceWarned:=false; viewportWarned:=false; xdataWarned:=false; xlineWarned:=false;
		HatchAsked := false; SplineAsked := False; CurveAsked := False; Line3DAsked := False; 
		PolygonMeshAsked:= False; PolyFaceMeshAsked:= False;
		EOFencountered := false;
		DonePrinterError := false;
		openBlocks := 0;
		
		
		if ThisIsDemo and OneDone then  {in Demo mode stop after 1 Conversion}
			Error(stringlist(StopDemoNr), 0, 0, true,false)
		else
			begin
				Convert_Cancelled := False;
				ACADversion := stringlist(unknownNoHeaderNr); {in case there's no header}
				convertdxf_file(droppedFileSpecs);
				HiliteMenu(0);
				OneDone := true;
			end;
		
		
		NewLogtext(currentFont, currentSize); {to update log window text}
	end;

{-----------------------------------------------------------}
end. {unit}