unit CTM;

{ ############  DXF TO ILLUSTRATOR #############################}
{    1991-1994 Universiteit Utrecht, FRW, Vakgroep Kartografie &                            }
{                           International Institute for Aerospace Survey And Earth Sciences ITC  }
{ 	Version 1.1.2  -  Main Parts Programmed by J.C. Blankert , 1991                        
		Version 2.2OS - code and app made OpenSource July 2002 BJK    }
{ ########################################################}

interface

	uses
		{UIP}
		fp,
		{MyUnites}
		ErrorsAndBeeps;

	type
		matrix = array[1..3, 1..3] of real;

	var
		IllCtm: matrix;
		ScrCtm: matrix;
	procedure initCTM (var CTM: Matrix);
	procedure translate (var CTM: Matrix; trx, try: real);
	procedure rotate (var CTM: Matrix; hoek: real);
	procedure scalectm (var CTM: Matrix; sx, sy: real);
	procedure transform (var CTM: Matrix; var x, y: real);
	procedure itransform (var CTM: Matrix; var x, y: real);

implementation
	var
		x, y: real;

	procedure mulctm (var CTM, mat1: matrix);
		var
			rij, kol, prd: integer;
			mat2: matrix;
	begin
		mat2 := ctm;
		ctm[1, 1] := 0;
		ctm[1, 2] := 0;
		ctm[1, 3] := 0;
		ctm[2, 1] := 0;
		ctm[2, 2] := 0;
		ctm[2, 3] := 0;
		ctm[3, 1] := 0;
		ctm[3, 2] := 0;
		ctm[3, 3] := 0;
		for rij := 1 to 3 do
			for kol := 1 to 3 do
				begin
					for prd := 1 to 3 do
						ctm[rij, kol] := ctm[rij, kol] + mat1[rij, prd] * mat2[prd, kol];
				end;
	end;

	procedure translate(var CTM: Matrix; trx, try: real);
		var
			mat: matrix;
	begin
		mat[1, 1] := 1;
		mat[1, 2] := 0;
		mat[1, 3] := 0;
		mat[2, 1] := 0;
		mat[2, 2] := 1;
		mat[2, 3] := 0;
		mat[3, 1] := trx;
		mat[3, 2] := try;
		mat[3, 3] := 1;
		mulctm(CTM, mat);
	end;

	procedure rotate;
		var
			mat: matrix;
	begin
		hoek := (pi * hoek) / 180;
		mat[1, 1] := cos(hoek);
		mat[1, 2] := sin(hoek);
		mat[1, 3] := 0;
		mat[2, 1] := -sin(hoek);
		mat[2, 2] := cos(hoek);
		mat[2, 3] := 0;
		mat[3, 1] := 0;
		mat[3, 2] := 0;
		mat[3, 3] := 1;
		mulctm(CTM, mat);
	end;

	procedure scalectm;
		var
			mat: matrix;
	begin
		mat[1, 1] := sx;
		mat[1, 2] := 0;
		mat[1, 3] := 0;
		mat[2, 1] := 0;
		mat[2, 2] := sy;
		mat[2, 3] := 0;
		mat[3, 1] := 0;
		mat[3, 2] := 0;
		mat[3, 3] := 1;
		mulctm(CTM, mat);
	end;

	procedure transform(var CTM: Matrix; var x, y: real);
		var
			xhulp: real;
	begin
		xhulp := ctm[1, 1] * x + ctm[2, 1] * y + ctm[3, 1];
		y := ctm[1, 2] * x + ctm[2, 2] * y + ctm[3, 2];
		x := xhulp;
	end;

	procedure itransform;
		var
			imat: matrix;
			det: real;
			rij, kolom, teken: integer;
			xhulp: real;
			r1, r2, k1, k2: integer;
	begin
		det := ctm[1, 1] * (ctm[2, 2] * ctm[3, 3] - ctm[2, 3] * ctm[3, 2]) - ctm[1, 2] * (ctm[2, 1] * ctm[3, 3] - ctm[2, 3] * ctm[3, 1]) + ctm[1, 3] * (ctm[2, 1] * ctm[3, 2] - ctm[2, 2] * ctm[3, 1]);
		teken := 1;
		for rij := 1 to 3 do
			begin
				if rij = 1 then
					begin
						r1 := 2;
						r2 := 3;
					end
				else if rij = 2 then
					begin
						r1 := 1;
						r2 := 3;
					end
				else
					begin
						r1 := 1;
						r2 := 2;
					end;
				for kolom := 1 to 3 do
					begin
						if kolom = 1 then
							begin
								k1 := 2;
								k2 := 3;
							end
						else if kolom = 2 then
							begin
								k1 := 1;
								k2 := 3;
							end
						else
							begin
								k1 := 1;
								k2 := 2;
							end;
						imat[kolom, rij] := teken * ((ctm[r1, k1] * ctm[r2, k2]) - (ctm[r1, k2] * ctm[r2, k1])) / det;
						teken := -teken;
					end;
			end;
		xhulp := imat[1, 1] * x + imat[2, 1] * y + imat[3, 1];
		y := imat[1, 2] * x + imat[2, 2] * y + imat[3, 2];
		x := xhulp;
	end;


	procedure initCTM;
	begin
		ctm[1, 1] := 1;
		ctm[1, 2] := 0;
		ctm[1, 3] := 0;
		ctm[2, 1] := 0;
		ctm[2, 2] := 1;
		ctm[2, 3] := 0;
		ctm[3, 1] := 0;
		ctm[3, 2] := 0;
		ctm[3, 3] := 1;
	end;

end.