unit Entities;

{ ############  DXF TO ILLUSTRATOR #############################}
{    1991-1994 Universiteit Utrecht, FRW, Vakgroep Kartografie &                            }
{                           International Institute for Aerospace Survey And Earth Sciences ITC  }
{ 	Version 1.1.2  -  Main Parts Programmed by J.C. Blankert , 1991                            }
{	Version 2.0   - programmed by B.J.Kbben																}
{						  - new routine read_XDATA (+change readEntities) - 1/94					}
{ 	Version 2.1 - BRUTE BUG FIX: removed dispose & exit here - 6/94							}
{	   		 - cleaned Bug Fix up: Warning & keep reading till normal group start (0)-8/94
		Version 2.2OS - code and app made OpenSource July 2002 BJK	}
{ ########################################################}

interface
	uses
		{UIP}
	QuickDraw,memory,windows,fp,QuickDrawText,fonts,menus,processes,
	Dialogs,Types,TextEdit,TextUtils,toolutils,
		{MyUnits}
			globals, ErrorsAndBeeps, myFiles, Setup,  myWindows, Layers, Collect, CTM;
	

	const
		MAXBLOCKS = 5000; {makes the handle 320 Kbytes}

	type
		NameStr = string[20];
		BlockInfo = record
				BlockName: NameStr;
				LayerName: NameStr;
				BlockBaseX: real;
				BlockBaseY: real;
				BlockBaseZ: real;
				BlockType: Integer;
				FilePos: Longint;
				DXFStartline : LongInt;
			end;
		DynamicBlocklist = array[1..MAXBLOCKS] of BlockInfo;
		BlockListPtr = ^DynamicBlocklist;
		BlockListHandle = ^BlockListPtr;

	var
		BlockList: BlockListHandle;
		
		

		
	function ReadEntities (why: reason): boolean;



implementation

	var
		startpos: longint;
		pnstate: PenState;
			
			
{----------------------------------------}
	procedure read_line (var group: grouptype);
		var
			sx, sy, sz, ex, ey, ez: real;
	begin
		repeat
			if not readgroup(group, normal) then
				exit(read_line);
			with group do
				case code of
					0: 
						;
					5, 39,100, 210,220, 230,999:
						; {ignore}
					1000..1071:
						begin
						if not XDATAwarned then
							begin
								warning(stringOf('XDATA ', stringlist(notImplNr)),0,true);
								XDATAwarned := TRUE;
							end;
						end;
					10: 
						sx := realdat;
					11: 
						ex := realdat;
					20: 
						sy := realdat;
					21: 
						ey := realdat;
					30: 
						sz := realdat;
					31: 
						ez := realdat;
					otherwise
						warning(stringof(stringList(UnexGroupTypeNr), ' [', code : 1, '] in LINE.'),DXFlineNr,true);
				end;
		until group.code = 0;
		Out_line(sx, sy, ex, ey); { dxf_line }
	end;
{----------------------------------------}
	procedure read_point (var group: grouptype);
		var
			x, y, z, xangle: real;
	begin
		xangle := 0;
		repeat
			if not readgroup(group, normal) then
				exit(read_point);
			with group do
				case code of
					0: 
						;
					5, 39,100, 210,220, 230,999:
						; {ignore}
					1000..1071:
						begin
						if not XDATAwarned then
							begin
								warning(stringOf('XDATA ', stringlist(notImplNr)),0,true);
								XDATAwarned := TRUE;
							end;
						end;
					10: 
						x := realdat;
					20: 
						y := realdat;
					30: 
						z := realdat;
					50: 
						xangle := realdat;
					otherwise
						warning(stringof(stringList(UnexGroupTypeNr), ' [', code : 1, '] in POINT.'),DXFlineNr,true);
				end;
		until group.code = 0;

		Out_Pixel(x, y);
	end;
{----------------------------------------}
	procedure read_circle (var group: grouptype);
		var
			x, y, z, rad: real;
	begin
		repeat
			if not readgroup(group, normal) then
				exit(read_circle);
			with group do
				case code of
					0: 
						;
					5, 39,100, 210,220, 230,999:
						; {ignore}
					1000..1071:
						begin
						if not XDATAwarned then
							begin
								warning(stringOf('XDATA ', stringlist(notImplNr)),0,true);
								XDATAwarned := TRUE;
							end;
						end;
					
					10: 
						x := realdat;
					20: 
						y := realdat;
					30: 
						z := realdat;
					40: 
						rad := realdat;
					otherwise
						warning(stringof(stringlist(UnexGroupTypeNR), ' [', code : 1, '] in CIRCLE.'),DXFlineNr,true);
				end;
		until group.code = 0;

		Out_Circle(x, y, rad);
	end;
{----------------------------------------}
	procedure read_arc (var group: grouptype);
		var
			x, y, z, rad, stangle, endangle: real;
	begin
		repeat
			if not readgroup(group, normal) then
				exit(read_arc);
			with group do
				case code of
					0: 
						;
					5:
						; {Handle: ignore}
					100:
						; {subclass marker: ignore}
					39,210,220,230,999:
						; {ignore}
					1000..1071:
						begin
						if not XDATAwarned then
							begin
								warning(stringOf('XDATA ', stringlist(notImplNr)),0,true);
								XDATAwarned := TRUE;
							end;
						end;
					
					10: 
						x := realdat;
					20: 
						y := realdat;
					30: 
						z := realdat;
					40: 
						rad := realdat;
					50: 
						stangle := realdat;
					51: 
						endangle := realdat;
					otherwise
						warning(stringof(stringList(UnexGroupTypeNr), ' [', code : 1, '] in ARC.'),DXFlineNr,true);
				end;
		until group.code = 0;
		Out_arc(x, y, rad, radial(stangle), radial(endangle));
	end;
	
{----------------------------------------}
	procedure read_solid (var group: grouptype);
		var
			x1, y1, x2, y2, x3, y3, x4, y4, zDump: real;
			box: boolean;

	begin
		box := false;
		repeat
			if not readgroup(group, normal) then
				exit(read_solid);
			with group do
				case code of
					0: 
						;
					
					5, 39,100, 210,220, 230,999:
						; {ignore}
					1000..1071:
						begin
						if not XDATAwarned then
							begin
								warning(stringOf('XDATA ', stringlist(notImplNr)),0,true);
								XDATAwarned := TRUE;
							end;
						end;
					8: 
						;
					10: 
						x1 := realdat;
					20: 
						y1 := realdat;
					30: 
						zDump := realdat; {z-coordinates not implemented}
					11: 
						x2 := realdat;
					21: 
						y2 := realdat;
					31: 
						zDump := realdat; {z-coordinates not implemented}
					12: 
						x3 := realdat;
					22: 
						y3 := realdat;
					32: 
						zDump := realdat; {z-coordinates not implemented}
					13: 
						begin
							box := true;
							x4 := realdat;
						end;
					23: 
						begin
							box := true;
							y4 := realdat;
						end;
					33: 
						begin
							zDump := realdat; {z-coordinates not implemented}
							box := true;
						end;
					otherwise
						warning(stringOf(stringlist(unSuppFldNr), ' [', code : 1, '] in SOLID.'),DXFlineNr,true);
				end;{case}
		until group.code = 0;
		Out_InitPolyLine(x1, y1, 0);
		Out_PolyLine(x3, y3, 0);
		if box then
			Out_PolyLine(x4, y4, 0);
		Out_PolyLine(x2, y2, 0);
		if SetUp^^.FillSolid then
			Out_ClosePolyLine(0, true, true)
		else
			Out_ClosePolyLine(0, true, false);
	end;
{----------------------------------------}
	procedure read_text (var group: grouptype);
		var
			theText: TextRecord;
	begin
		with TheText do
			begin
				scale := 1;
				angle := 0;
				oblangle := 0;
				textgenflags := 0;
				justtype := 0;
				tekst := '';
				font := 'STANDARD'; {because some DXF translators omit this default font}
				height := 1;
				repeat
					if not readgroup(group, normal) then
						exit(read_text);
					with group do
						case code of
							0: 
								;
							5, 39,100, 210,220, 230,999:
								; {ignore}
							1000..1071:
								begin
								if not XDATAwarned then
									begin
										warning(stringOf('XDATA ', stringlist(notImplNr)),0,true);
										XDATAwarned := TRUE;
									end;
								end;
							1: 
								tekst := strdat^;
							7: 
								font := strdat^;
							10: 
								x := realdat;
							11: 
								alx := realdat;
							20: 
								y := realdat;
							21: 
								aly := realdat;
							30: 
								z := realdat;
							31: 
								alz := realdat;
							40: 
								height := realdat;
							41: 
								scale := realdat;
							50: 
								angle := realdat;
							51: 
								oblangle := realdat;
							71: 
								textgenflags := intdat;
							72: 
								justtype := intdat; { 0:left, 1:centred, 2:right, 3:heightalign}
                                             { 4:fully centred, 5: betweenpoints(width varies)}
							73: 
								; {vertical alignment (AutoCad 11) ignore for now}
							otherwise
								warning(stringOf(stringlist(unSuppFldNr), ' [', code : 1, '] in TEXT.'),DXFlineNr,true);
						end; {case}
				until group.code = 0;
			end;

		Out_Text(theText);
	end;
	
{----------------------------------------}
	procedure read_insert (var group: grouptype);
		var
			colspace, rowspace, blockangle: real;
			tel, attrfollowflag, cols, rows: integer;
			XBase, YBase, ZBase, XScalefac, YScaleFac, ZScaleFac: real;
			OldScrCtm, OldIllCtm, ScrCRctm, DxfCrctm: Matrix;
			CurrentFilePos,BlockFilePos,CurrentDXFLine: Longint;
			ctel, rtel: integer;
	begin
		Hlock(Handle(BlockList));
		xbase := 0;
		ybase := 0;
		zbase := 0;
		XScaleFac := 1;
		YScaleFac := 1;
		ZScaleFac := 1;
		colspace := 0;
		rowspace := 0;
		Blockangle := 0;
		attrfollowflag := 0;
		cols := 1;
		rows := 1;
		repeat {until group.code = 0}
			if not readgroup(group, normal) then
				exit(read_insert);
			with group do
				case code of
					0: 
						;
					
					5,100, 210,220, 230,999:
						; {ignore}
					1000..1071:
						begin
						if not XDATAwarned then
							begin
								warning(stringOf('XDATA ', stringlist(notImplNr)),0,true);
								XDATAwarned := TRUE;
							end;
						end;
					2: 
						begin
							tel := 1;
							while (tel < BlockCount) and (BlockList^^[tel].Blockname <> strdat^) do
								tel := tel + 1;
							if tel > BlockCount then
								begin
									Error(stringOf(stringlist(BlockErrNr), '', strdat^, ' '), NotFoundNr, 0, true, false);
									exit(read_insert);
								end;
						end;
					8: 
						;
					10: 
						xbase := realdat - BlockList^^[tel].BlockBaseX;
					20: 
						ybase := realdat - BlockList^^[tel].BlockBaseY;
					30: 
						Zbase := realdat - BlockList^^[tel].BlockBaseZ;
					41: 
						Xscalefac := realdat;
					42: 
						Yscalefac := realdat;
					43: 
						Zscalefac := realdat;
					44: 
						colspace := realdat;
					45: 
						rowspace := realdat;
					50: 
						Blockangle := realdat;
					66: 
						attrfollowflag := intdat;
					70: 
						cols := intdat;
					71: 
						rows := intdat;
					otherwise
						;
				end; {case}
		until group.code = 0;
		
		BlockFilePos := (BlockList^^[tel].FilePos);
		HUnlock(Handle(BlockList));
		
		CurrentFilePos := GetFilePos;
		OldIllCtm := IllCtm;
		Translate(IllCtm, Xbase, Ybase);
		scaleCtm(IllCtm, XscaleFac, YscaleFac);
		Rotate(IllCtm, BlockAngle);
		OldScrCtm := ScrCtm;
		Translate(ScrCtm, Xbase, Ybase);
		scaleCtm(ScrCtm, XscaleFac, YscaleFac);
		Rotate(ScrCtm, BlockAngle);
		ScrCRCtm := ScrCtm;
		DxfCRCtm := IllCtm;
		for ctel := 1 to cols do
			for rtel := 1 to rows do
				begin
					ScrCtm := ScrCRCtm;
					IllCtm := DxfCRCtm;
					SetFilePos(BlockFilePos);
					translate(IllCtm, (ctel - 1) * colspace, (rtel - 1) * rowspace);
					translate(IllCtm, (ctel - 1) * colspace, (rtel - 1) * rowspace);
					translate(ScrCtm, (ctel - 1) * colspace, (rtel - 1) * rowspace);
					translate(ScrCtm, (ctel - 1) * colspace, (rtel - 1) * rowspace);
					
					
					if SetUp^^.GroupBlocks then
						begin
						itsABlock := true;
						Out_Begin_Block;
						end;
						
					CurrentDXFLine := DXFLineNr;
					DXFLineNr := BlockList^^[tel].DXFStartLine;
					if not ReadEntities(readingInsert) then
						begin
							ScrCtm := OldScrCtm;
							IllCtm := OldIllCtm;
							SetFilePos(CurrentFilePos);
							exit(Read_Insert);
						end;
					DXFLineNr := CurrentDXFLine;
						
					if SetUp^^.GroupBlocks then
						begin
						itsABlock := false;
						Out_End_Block;
						end;
										
				end;
		IllCtm := OldIllCtm;
		ScrCtm := OldScrCtm;
		SetFilePos(CurrentFilePos);
	end;
	
{----------------------------------------}
	procedure read_polyline (var group: grouptype);
		var
			followflag: boolean;
			x, y, z, bulge, nextbulge, curvefitangledir: real;
			localswidth, localewidth, startwidth, endwidth: real;
			first: boolean;
			polylnflag, vertexcount, SmoothM, SmoothN, MeshCount: integer;
			vertexflag: integer;
			closed, skip: boolean;
			regel: string;
	begin
		skip := False;
		polylnflag := 0;
		vertexcount := 0;
		startwidth := 0;
		smoothM := 0;
		smoothN := 0; {default values}
		followflag := FALSE;
		repeat
			if not readgroup(group, normal) then
				exit(read_polyline);
			with group do
				case code of
					0: 
						;
					
					5, 39,100, 210,220, 230,999:
						; {ignore}
					1000..1071:
						begin
						if not XDATAwarned then
							begin
								warning(stringOf('XDATA ', stringlist(notImplNr)),0,true);
								XDATAwarned := TRUE;
							end;
						end;
					75:
						; { sort of smooth surface: ignore}
					10: 
						x := realdat;
					20: 
						y := realdat;
					30: 
						z := realdat; 
					66: 
						begin
							if intdat <> 1 then
								warning(stringlist(VerticesFollowNr),DXFlineNr,true);
							followflag := TRUE;
						end;
					70: 
						polylnflag := intdat;
					40: 
						startwidth := realdat;
					41: 
						endwidth := realdat;
					71: 
						meshcount := intdat;
					72: 
						vertexcount := intdat;
					73: 
						SmoothM := intdat;
					74: 
						smoothN := intdat;
					otherwise
						warning(stringof(stringlist(UnexGroupTypeNr), ' [', code : 1, '] in POLYLINE.'),DXFlineNr,true);
				end;
		until group.code = 0;
		
		closed := (bitand(polylnflag, 1) = 1);
		if bitand(polylnflag, 2) = 2 then { polyline bevat curve fit punten }
			begin
				skip := not ask_curve_fit;
			end
		else if bitand(polylnflag, 4) = 4 then {polyline bevat spline punten }
			begin
				skip := not ask_spline_fit;
			end
		else if bitand(polylnflag, 8) = 8 then { 3D polyline }
			begin
				skip := not ask_3D_line;
			end
		else if bitand(polylnflag, 16) = 16 then { 3D polygon mesh}
			begin
				skip := not ask_polygonMesh;
			end
		else if bitand(polylnflag, 64) = 64 then { polyface mesh }
			begin
				skip := not ask_polyfaceMesh;
			end;
		
		if skip then
			begin
				repeat
					if not readline(DxfFileNo, regel) then
						exit(Read_PolyLine);
				until regel = 'SEQEND';
			end
			
		else  {NORMAL, or try out anyway if one of the above polyflags is true}
			begin
				first := TRUE;
				if followflag then
					with group do
						repeat
							localswidth := startwidth;
							localewidth := endwidth;
							nextbulge := 0;
							vertexflag := 0;
							repeat
								if not readgroup(group, normal) then
									exit(read_polyline);
								if aborted then
									exit(read_polyline);
								case code of
									0: 
										;
									5, 39,100, 210,220, 230,999:
										; {ignore}
									1000..1071:
										begin
										if not XDATAwarned then
											begin
												warning(stringOf('XDATA ', stringlist(notImplNr)),0,true);
												XDATAwarned := TRUE;
											end;
										end;
									10: 
										x := realdat;
									20: 
										y := realdat;
									30: 
										z := realdat;
									40: 
										localswidth := realdat;
									41: 
										localewidth := realdat;
									42: 
										nextbulge := realdat;
									70: 
										vertexflag := intdat;
									50: 
										curvefitangledir := realdat;
									71,72,73,74:
										; {polyface mesh vertice indexes: ignore}
									otherwise
										warning(stringOf(stringlist(unSuppFldNr), ' [', code : 1, '] in VERTEX.'),DXFlineNr,true);
								end; {case}
							until code = 0;
							if vertexflag <> 0 then {curve, spline of 3d punt }
								nextbulge := 0;
							if first then
								begin
									Out_InitPolyLine(x, y, startwidth);
									bulge := nextbulge;
									first := FALSE;
								end
							else
								begin
									Out_PolyLine(x, y, bulge);
									bulge := nextbulge;
								end;
						until strdat^ = 'SEQEND';
				if SetUp^^.FillPoly then
					Out_ClosePolyLine(bulge, closed, true)
				else
					Out_ClosePolyLine(bulge, closed, false);
			end; { not skip }
		if not readgroup(group, normal) then
			exit(read_polyline);
	end;

{----------------------------------------}
	procedure read_Threedface (var group: grouptype);
	begin
		repeat
			if not readgroup(group, normal) then
				exit(read_Threedface);
		until group.code = 0;
		if not Threedfacewarned then
			begin
				warning(stringOf('3DFACE ', stringlist(notImplNr)),0,true);
				Threedfacewarned := true;
			end;
	end;
{----------------------------------------}
	procedure read_Threedsolid (var group: grouptype);
	begin
		repeat
			if not readgroup(group, normal) then
				exit(read_Threedsolid);
		until group.code = 0;
		if not Threedsolidwarned then
			begin
				warning(stringOf('3DSOLID ', stringlist(notImplNr)),0,true);
				ThreedsolidWarned := true;
			end;
	end;
{----------------------------------------}
	procedure read_body (var group: grouptype);
	begin
		repeat
			if not readgroup(group, normal) then
				exit(read_body);
		until group.code = 0;
		if not bodywarned then
			begin
				warning(stringOf('BODY ', stringlist(notImplNr)),0,true);
				bodywarned := true;
			end;
	end;
	{----------------------------------------}
	procedure read_ellipse (var group: grouptype);
	begin
		repeat
			if not readgroup(group, normal) then
				exit(read_ellipse);
		until group.code = 0;
		if not ellipsewarned then
			begin
				warning(stringOf('ELLIPSE ', stringlist(notImplNr)),0,true);
				ellipsewarned := true;
			end;
	end;
	{----------------------------------------}
	procedure read_leader (var group: grouptype);
	begin
		repeat
			if not readgroup(group, normal) then
				exit(read_leader);
		until group.code = 0;
		if not leaderwarned then
			begin
				warning(stringOf('LEADER ', stringlist(notImplNr)),0,true);
				leaderwarned := true;
			end;
	end;
	{----------------------------------------}
	procedure read_mline (var group: grouptype);
	begin
		repeat
			if not readgroup(group, normal) then
				exit(read_mline);
		until group.code = 0;
		if not mlinewarned then
			begin
				warning(stringOf('MLINE ', stringlist(notImplNr)),0,true);
				mlinewarned := true;
			end;
	end;
	{----------------------------------------}
	procedure read_mtext (var group: grouptype);
	begin
		repeat
			if not readgroup(group, normal) then
				exit(read_mtext);
		until group.code = 0;
		if not mtextwarned then
			begin
				warning(stringOf('MTEXT ', stringlist(notImplNr)),0,true);
				mtextwarned := true;
			end;
	end;
	{----------------------------------------}
	procedure read_oleframe (var group: grouptype);
	begin
		repeat
			if not readgroup(group, normal) then
				exit(read_oleframe);
		until group.code = 0;
		if not oleframewarned then
			begin
				warning(stringOf('OLEFRAME ', stringlist(notImplNr)),0,true);
				oleframewarned := true;
			end;
	end;
	{----------------------------------------}
	procedure read_ray (var group: grouptype);
	begin
		repeat
			if not readgroup(group, normal) then
				exit(read_ray);
		until group.code = 0;
		if not raywarned then
			begin
				warning(stringOf('RAY ', stringlist(notImplNr)),0,true);
				raywarned := true;
			end;
	end;
	{----------------------------------------}
	procedure read_region (var group: grouptype);
	begin
		repeat
			if not readgroup(group, normal) then
				exit(read_region);
		until group.code = 0;
		if not regionwarned then
			begin
				warning(stringOf('REGION ', stringlist(notImplNr)),0,true);
				regionwarned := true;
			end;
	end;
	{----------------------------------------}
	procedure read_spline (var group: grouptype);
	begin
		repeat
			if not readgroup(group, normal) then
				exit(read_spline);
		until group.code = 0;
		if not splinewarned then
			begin
				warning(stringOf('SPLINE ', stringlist(notImplNr)),0,true);
				splinewarned := true;
			end;
	end;
	{----------------------------------------}
	procedure read_tolerance (var group: grouptype);
	begin
		repeat
			if not readgroup(group, normal) then
				exit(read_tolerance);
		until group.code = 0;
		if not tolerancewarned then
			begin
				warning(stringOf('TOLERANCE ', stringlist(notImplNr)),0,true);
				tolerancewarned := true;
			end;
	end;
	{----------------------------------------}
	procedure read_viewport (var group: grouptype);
	begin
		repeat
			if not readgroup(group, normal) then
				exit(read_viewport);
		until group.code = 0;
		if not viewportwarned then
			begin
				warning(stringOf('VIEWPORT ', stringlist(notImplNr)),0,true);
				viewportwarned := true;
			end;
	end;
	{----------------------------------------}
	procedure read_xline (var group: grouptype);
	begin
		repeat
			if not readgroup(group, normal) then
				exit(read_xline);
		until group.code = 0;
		if not xlinewarned then
			begin
				warning(stringOf('XLINE ', stringlist(notImplNr)),0,true);
				xlinewarned := true;
			end;
	end;
{----------------------------------------}
	procedure read_trace (var group: grouptype);
	begin
		repeat
			if not readgroup(group, normal) then
				exit(read_trace);
		until group.code = 0;
		if not TRACEwarned then
			begin
				warning(stringOf('TRACE ', stringlist(notImplNr)),0,true);
				TRACEWarned := true;
			end;
	end;
{----------------------------------------}
	procedure read_shape (var group: grouptype);
	begin
		repeat
			if not readgroup(group, normal) then
				exit(read_shape);
		until group.code = 0;
		if not SHAPEwarned then
			begin
				warning(stringOf('SHAPE ', stringlist(notImplNr)),0,true);
				SHAPEWarned := true;
			end;
	end;

{----------------------------------------}
	procedure read_attdef (var group: grouptype);
	begin
		repeat
			if not readgroup(group, normal) then
				exit(read_attdef);
		until group.code = 0;
		if not ATTDEFwarned then
			begin
				warning(stringOf('ATTDEF ', stringlist(notImplNr)),0,true);
				ATTDEFWarned := true;
			end;
	end;
{----------------------------------------}
	procedure read_attrib (var group: grouptype);
	begin
		repeat
			if not readgroup(group, normal) then
				exit(read_attrib);
			if group.strdat^ = 'SEQEND' then
				if not readgroup(group, normal) then
					exit(read_attrib);
		until group.code = 0;
		if not ATTRIBwarned then
			begin
				warning(stringOf('ATTRIB ', stringlist(notImplNr)),0,true);
				ATTRIBWarned := true;
			end;
	end;
{----------------------------------------}
	procedure read_dimension (var group: grouptype);
	begin
		repeat
			if not readgroup(group, normal) then
				exit(read_dimension);
		until group.code = 0;
		if not dimensionwarned then
			begin
				warning(stringOf('DIMENSION ', stringlist(notImplNr)),0,true);
				dimensionwarned := TRUE;
			end;
	end;


{----------------------------------------}
	procedure read_XDATA (var group: grouptype);
			
	begin
		if not XDATAwarned then
			begin
				warning(stringOf('XDATA ', stringlist(notImplNr)),0,true);
				XDATAwarned := TRUE;
			end;
		repeat
			if not readgroup(group, normal) then
				exit(read_XDATA);
		until group.code = 0;
	end;

{----------------------------------------}
	procedure read_SeqEnd (var group: grouptype);
			
	begin
		warning(stringOf('SEQEND ', stringlist(UnexpectedPlaceNr)),DXFlineNr,true);
		repeat
			if not readgroup(group, normal) then
				exit(read_SeqEnd);
		until group.code = 0;
	end;
{----------------------------------------}
	procedure read_vertex (var group: grouptype);
			
	begin
		warning(stringOf('VERTEX ', stringlist(notImplNr)),0,true);
		repeat
			if not readgroup(group, normal) then
				exit(read_vertex);
		until group.code = 0;
	end;
{----------------------------------------}
	procedure read_unknown (var group: grouptype);
	begin
		warning(concat(stringlist(unknownEntityNr), '', group.strdat^, ' '),0,true);
		repeat
			if not readgroup(group, normal) then
				exit(read_unknown);
		until group.code = 0;
	end;
	
{----------------------------------------}
	function ReadEntities (why: reason): boolean;
		var
			group: grouptype;

	begin
		
		if not MemAvail(sizeof(strtype)) then 
			Error('[ReadEntities:1] ', OutOfMemNr, 0, true,true);
		group.strdat := strptr(newPtr(sizeof(strtype)));
		group.strdat^ := '';
		ReadEntities := False;
		if not readgroup(group, normal) then
			begin
				if group.strdat <> nil then disposePtr(Ptr(group.strdat));
				exit(ReadEntities);
			end;
			
		repeat {until group.strdat^ = 'ENDSEC'}
		
			if (group.code <> 0) then	{something other then name of entity => find out what}
			
				begin
				case group.code of
					5:
						; {Handle: Ignore}
					100:
						; {Subclass marker: Ignore}
					6,8,48,60,62,67,999:
						; {Various other ones, are dealt with in readgroup procedures if necessary}
					1000..1071:
						begin
						if not XDATAwarned then
							begin
								warning(stringOf('XDATA ', stringlist(notImplNr)),DXFlineNr,true);
								XDATAwarned := TRUE;
							end;
						end;
					otherwise {all other ones : ignore}
						;{ ignore}
					end; {case}
					if not readgroup(group, normal) then
						exit(ReadEntities);
				end
			
			else {grouptype = 0}
			
				begin 
				if not MemAvail(10000) then
					begin
						Error('[ReadEntities:2]', OutOfMemNr, DXFLineNr, true,true);
						ReadEntities := True;
						disposePtr(Ptr(group.strdat));
						exit(ReadEntities);
					end;
				if why = normal then
					doBalk(GetfilePos);
				if aborted then
					begin
						ReadEntities := True;
						disposePtr(Ptr(group.strdat));
						aborted := false;
						exit(readEntities);
					end;
				if group.strdat^ = '3DFACE' then
					read_Threedface(group)		
				else if group.strdat^ = '3DSOLID' then
					read_Threedsolid(group)
				else if group.strdat^ = 'ARC' then
					read_arc(group)
				else if group.strdat^ = 'ATTDEF' then
					read_attdef(group)
				else if group.strdat^ = 'ATTRIB' then
					read_attrib(group)
				else if group.strdat^ = 'BODY' then
					read_body(group)
				else if group.strdat^ = 'CIRCLE' then
					read_circle(group)
				else if group.strdat^ = 'DIMENSION' then
					read_dimension(group)	
				else if group.strdat^ = 'ELLIPSE' then
					read_ellipse(group)	
				else if group.strdat^ = 'INSERT' then
					read_insert(group)
				else if group.strdat^ = 'LEADER' then
					read_leader(group)
				else if group.strdat^ = 'LINE' then
					read_line(group)
				else if group.strdat^ = 'MLINE' then
					read_mline(group)
				else if group.strdat^ = 'MTEXT' then
					read_MTEXT(group)
				else if group.strdat^ = 'OLEFRAME' then
					read_OLEFRAME(group)
				else if group.strdat^ = 'POINT' then
					read_point(group)
				else if group.strdat^ = 'POLYLINE' then
					read_polyline(group)
				else if group.strdat^ = 'RAY' then
					read_ray(group)
				else if group.strdat^ = 'REGION' then
					read_region(group)
				else if group.strdat^ = 'SEQEND' then
					read_SEQEND(group)
				else if group.strdat^ = 'SHAPE' then
					read_shape(group)
				else if group.strdat^ = 'SOLID' then
					read_solid(group)
				else if group.strdat^ = 'SPLINE' then
					read_SPLINE(group)
				else if group.strdat^ = 'TEXT' then
					read_text(group)
				else if group.strdat^ = 'TOLERANCE' then
					read_TOLERANCE(group)
				else if group.strdat^ = 'TRACE' then
					read_trace(group)
				else if group.strdat^ = 'VERTEX' then
					read_VERTEX(group)
				else if group.strdat^ = 'VIEWPORT' then
					read_VIEWPORT(group)
				else if group.strdat^ = 'XDATA' then
					read_XDATA(group)
				else if group.strdat^ = 'XLINE' then
					read_XLINE(group)
				else if group.strdat^ = 'ENDBLK' then {was reading entities within block def => exit}
					begin
						ReadEntities := True;
						disposePtr(Ptr(group.strdat));
						exit(ReadEntities); { ReadEntities vanuit INSERT aangeroepen, dus terug naar daar}
					end
				else
					read_unknown(group);
					
				end;
		until group.strdat^ = 'ENDSEC';
		
		
		ReadEntities := True;
		disposePtr(Ptr(Group.strdat));
			
	end; {of function readentities}

{----------------------------------------}

end. { of unit entities}