unit layers;


{ ############  DXF TO ILLUSTRATOR #############################}
{    1991-1994 Universiteit Utrecht, FRW, Vakgroep Kartografie &                            }
{                           International Institute for Aerospace Survey And Earth Sciences ITC  }
{ 	Version 1.1.2  -  Main Parts Programmed by J.C. Blankert , 1991                            }
{ 	Version 1.2.0  -  Changes Programmed by B.J. Kbben, 20-08-92                              }
{ 	                 (TieLines boolean determines if lines are tied together in proc. PrintElems)}
{						-Bug Fix: If special character '\' in text, change to '\\'	- BJK dec 92		}
{ 	Version 2.0     -  Changes programmed by B.J. Kbben, november/december 1992      }
{ 	 							- Scaling: IllMaxX and Y are now reals instead of constants            }
{ 	 											These are set in the new unit Scale					            }
{ 	Version 2.1     -  Changes programmed by B.J. Kbben, june-august 1994  				   }
{ 	 							- 5.0 Layer information														           }
{ 	 							- Font, lineweight & Color Substitution								           }
{							- added  'u' & 'U' in Printelements for grouping blocks - 8/94	
		Version 2.2OS - code and app made OpenSource July 2002 BJK	}
{ ########################################################}

interface
	
	uses
		{UIP}
	QuickDraw,memory,windows,fp,QuickDrawText,fonts,menus,processes,
	Dialogs,Types,TextEdit,TextUtils,controls,toolutils,resources,
			{MyUnits}
			globals, errorsandbeeps,log, Scale, SetUp, Ctm, myFiles,myWindows,tables;
	

	var
	
		CurrentIllustratorFont, CurrentIllColor: str255;	

	procedure InitLayers;
	procedure DisPoseLayers;
	procedure PrintLayers (maxx, maxy: real);
	procedure SetNewLayer (LayerName: NameString);
	procedure SetNewLType (LTypeName: NameString);
	procedure SetNewColor (ColorNum: Integer);
	procedure AddElem (Etype: char; p1, p2, p3: punttype);
	procedure AddTekst (var theText: TextRecord);
	function LayerCount: integer;
	function radial (graden: real): real;
	procedure printNote (note: string);

implementation


	type
		ElemHandle = ^ElemTypePtr;
		ElemTypePtr = ^ElemType;
		ElemType = record
				ElemName: char;
				x1, y1, x2, y2, x3, y3: real;
				next: ElemTypePtr;
			end;
			
		ColorHandle = ^ColorTypePtr;
		ColorTypePtr = ^ColorType;
		ColorType = record
				ColorNum: integer;
				ElemData: ElemTypePtr;
				CurElem: ElemHandle;
				next: ColorTypePtr;
			end;
			
		LTypeHandle = ^LtypeTypePtr;
		LTypeTypePtr = ^LTypeType;
		LTypeType = record
				LTypeName: NameString;
				ColorData: ColorTypePtr;
				CurColor: ColorHandle;
				next: LTypeTypePtr;
			end;
		TextHandle = ^TextTypePtr;
		TextTypePtr = ^TextType;
		TextType = record
				alignment: integer;
				TextData: string;
				matrixstring: string;
				next: TextTypePtr;
			end;
		FontListPtr = ^FontList;
		FontList = record
				FontName: NameString;
				TextData: TextTypePtr;
				CurText: TextHandle;
				next: FontListPtr;
			end;
		LayerHandle = ^LayerTypePtr;
		LayerTypePtr = ^LayerType;
		LayerType = record
				LayerName: NameString;
				Fonts: FontListPtr;
				LTypeData: LTypeTypePtr;
				CurLtype: LTypeHandle;
				Next: LayerTypePtr;
			end;

	var
		root: LayerTypePtr;
		CurLayer: LayerHandle;

		textx, texty: real;
		layerColor: integer;
		
	{---------------------------}
procedure printNote (note: string);
		var
			mtxstring: string;
			textCtm: Matrix;
			height: real;
	begin
		height := 10 / illscale;
		textCtm := IllCtm;
		translate(textCtm, textx, texty);
		scalectm(textctm, 1 / illscale, 1 / illscale);
		if not WritelnString(IllFileNo, StringOf('/_Courier 10 10 0 0 z ')) then
			exit(PrintNote);
		mtxstring := stringof('[', textctm[1, 1] : 4 : 4, ' ', textctm[1, 2] : 4 : 4, ' ');
		mtxstring := concat(mtxstring, stringof(textctm[2, 1] : 4 : 4, ' ', textctm[2, 2] : 4 : 4, ' '));
		mtxstring := concat(mtxstring, stringof(textctm[3, 1] : 4 : 4, ' ', textctm[3, 2] : 4 : 4, ']e'));
		if not WritelnString(IllFIleNo, mtxstring) then
			exit(PrintNote);
		if not WritelnString(IllFileNo, stringof('(', note, ')t')) then
			exit(PrintNote);
		if not WritelnString(IllFileNo, 'T') then
			exit(PrintNote);
		texty := texty - height;
	end;

	{---------------------------}
function radial (graden: real): real;
	begin
		radial := graden * pi / 180;
	end;

	{---------------------------}
function GetDashString (var LtypeName: string): string;
		var
			tel: integer;
			lasttype: boolean;
	begin
		lasttype := False;
		GetDashString := '[]0 d';
		tel := 0;
		repeat
			tel := tel + 1;
			if tel > maxltypes then
				begin
					tel := tel - 1;
					lasttype := true;
				end;
		until (LtypeDash[tel].LtypeName = LtypeName) or (LtypeDash[tel].LtypeName = '') or lasttype;
		if LtypeDash[tel].LtypeName = LtypeName then
			GetDashString := LtypeDash[tel].DashString;
	end;

	{---------------------------}
procedure NewElemType (var Elem: ElemTypePtr);
	begin
		if not MemAvail(sizeof(ElemType)) then 
			Error('[NewElemType] ', OutOfMemNr, 0, true,true);
		Elem := ElemTypePtr(newPtr(sizeof(ElemType)));
		with Elem^ do
			begin
				ElemName := char(0);
				x1 := 0;
				y1 := 0;
				x2 := 0;
				y2 := 0;
				x3 := 0;
				y3 := 0;
				next := nil;
			end;
	end;	

	{---------------------------}
procedure DisposeElems (Root: ElemTypePtr); { niet recursief, want zeeeer groot }

		var
			NextPtr: ElemTypePtr;
	begin
		while Root <> nil do
			begin
				NextPtr := Root^.next;
				disposePtr(Ptr(Root));
				Root := NextPtr;
			end;
	end;

	{---------------------------}
procedure PrintElems (Root: ElemTypePtr); { niet recursief, want zeeeer groot }
		var
			tempstr: string;
			firstx, firsty, lastx, lasty: real;
			skipmoveto: boolean;
			stillNeed : char;
			vertexcount: integer;

	begin
		skipmoveto := False;
		vertexcount := 0;
		stillNeed := '0';
		if Root <> nil then
			begin
				repeat {until root = nil;}
					with root^ do
						begin
						
							case ElemName of
								'c': 
									begin
										tempstr := stringof(x1 : 5 : 4, ' ', y1 : 5 : 4, ' ', x2 : 5 : 4, ' ', y2 : 5 : 4, ' ', x3 : 5 : 4, ' ', y3 : 5 : 4, ' ');
										lastx := x3;
										lasty := y3;
										vertexcount := vertexcount + 3;
									end;
								'w': 
									begin
										if stillNeed <> '0' then
											begin
												if not WritelnString(IllFileNo, concat(stillNeed,' %block begin/end ')) then
													exit(PrintElems);
												stillNeed := '0';
											end;
										tempstr := stringOf(x1 : 5 : 4, ' ');
									end;
								'W':  {is not valid Illustrator Format => just a flag to reset linewidth}
									begin
										if stillNeed <> '0' then
											begin
												if not WritelnString(IllFileNo, concat(stillNeed,' %block begin/end ')) then
													exit(PrintElems);
												stillNeed := '0';
											end;
										tempstr := stringOf(DefaultLW, ' w %reset line width ');
									end;
								'm': 
									begin
										if (SetUp^^.Tielines) then 
											begin
												if (x1 = lastx) and (y1 = lasty) then
													begin
														skipmoveto := true;
													end
												else
													begin
														skipmoveto := false;
														if stillNeed <> '0' then
															if WritelnString(IllFileNo, concat(stillNeed,' %Not joined: XY-first <> XY-last')) then
																stillNeed := '0'
															else
																exit(PrintElems);
														vertexcount := 0;
														tempstr := stringof(x1 : 5 : 4, ' ', y1 : 5 : 4, ' ');
														firstx := x1;
														firsty := y1;
													end;
											end
										else
											begin
												vertexcount := 0;
												tempstr := stringof(x1 : 5 : 4, ' ', y1 : 5 : 4, ' ');
												firstx := x1;
												firsty := y1;
											end;
									end;
								'l': 
									begin
										vertexcount := vertexcount + 1;
										tempstr := stringof(x1 : 5 : 4, ' ', y1 : 5 : 4, ' ');
										lastx := x1;
										lasty := y1;
									end;
								'u': 
									begin
										if stillNeed <> '0' then
											begin
												if not WritelnString(IllFileNo, concat(stillNeed,' %block begin')) then
													exit(PrintElems);
												stillNeed := '0';
											end;
										tempStr := '';
									end;
								'U': 
									begin
										if stillNeed <> '0' then
											begin
												if not WritelnString(IllFileNo, concat(stillNeed,' %block end')) then
													exit(PrintElems);
												stillNeed := '0';
											end;
										tempStr := '';
									end;
							 'b': {check Tie, else write the letter}
									begin
										if (SetUp^^.Tielines) then { 'm' takes care of it}
											begin
												tempstr := '%Join? -';
												stillNeed := 'b';
											end
										else
											tempstr := '';
									end; {'b'}
								'S': {check Tie, else write the letter}
									begin
										if (SetUp^^.Tielines) then { 'm' takes care of it}
											begin
												tempstr := '%Join? -';
												stillNeed := 'S';
											end
										else
											tempstr := '';
									end; {'S'}
								's': {check Tie, else write the letter}
									begin
										if (SetUp^^.Tielines) then { 'm' takes care of it}
											begin
												tempstr := '%Join? -';
												stillNeed := 's';
											end
										else
											tempstr := '';
									end; {'s'}
							end; { case }
							if (skipmoveto) then {skip 'm'}
								begin
									if not WritelnString(IllFileNo, '%Made a Join') then
										exit(PrintElems);
									skipmoveto := false;
								end
							else
								begin
									if WriteString(IllFileNo, tempstr) then
										if WritelnString(IllFileNo, ElemName) then
											{OK}
										else
											exit(PrintElems);
								end;
						end; {with root^}
					
					{}numElems := numElems +1; {to keep count of progress}
					if numElems/30 = roundtol(numElems/30) then
							DoBalk(numElems);
					
					Root := root^.next;
					if (vertexcount > 950) and (root <> nil) then
						begin
							if SetUp^^.TargetAppl = Freehand then { SetUp for FreeHand ?}
								begin
									if not WriteLnString(IllFileNo, 'S %FREEHAND SPLIT') then
										exit(PrintElems);
									if not WritelnString(IllFileNo, stringof(lastx : 3 : 4, ' ', lasty : 3 : 4, ' m')) then
										exit(printelems);
								end;
							vertexcount := 0;
						end;
				until root = nil;
			end;
		if SetUp^^.Tielines and (StillNeed <> '0') then {clean up when Joining lines}
			begin
			if not WritelnString(IllFileNo, concat(stillNeed,' %Cleaned up Joining ')) then
				exit(PrintElems);
			stillNeed := '0';
			end;
	end;


	{---------------------------}
procedure NewColor (var Color: ColorTypePtr);
	begin
		if not MemAvail(sizeof(ColorType)) then 
			Error('[NewColor] ', OutOfMemNr, 0, true,true);
		
		Color := ColorTypePtr(newPtr(sizeof(ColorType)));
		with Color^ do
			begin
				ColorNum := 0;
				ElemData := nil;
				CurElem := @ElemData;
				next := nil;
			end;
	end;

	{---------------------------}
procedure DisposeColors (Root: ColorTypePtr);
		var
			NextPtr: ColorTypePtr;
	begin
		while root <> nil do
			begin
				DisposeElems(root^.ElemData);
				NextPtr := root^.next;
				disposePtr(Ptr(root));
				Root := NextPtr;
			end;
	end;

	{---------------------------}
function ColorHasData (Color: ColorTypePtr): boolean;
	begin
		if Color = nil then
			begin
				ColorHasData := False;
				exit(ColorHasData);
			end;
		ColorHasData := (Color^.ElemData <> nil);
	end;

	{---------------------------}
procedure PrintColors (Root: ColorTypePtr);
		var
			FoundColor, i, LP: integer;
			FoundIt: boolean;
	begin
		while root <> nil do
			begin
				if Root^.Elemdata <> nil then
					begin
						if SetUp^^.Grouping = full then   {only then make Color a Group}
							if not WritelnString(IllFileNo, 'u %colour grouping') then
								exit(PrintColors);
						if SetUp^^.ColorSubstitution then
							begin
								CurrentIllColor := '0 0 0 1'; {if its not in DXF list}
								FoundIt := false;
								for i := 1 to maxNumFonts do
									begin
										LP := pos('\', Setup^^.ColorTable[i]);
										FoundColor := -9999;
										readstring(copy(Setup^^.ColorTable[i], 1, LP - 1), FoundColor);
										if FoundColor = root^.colornum then
											begin
												FoundIt := true;
												CurrentIllColor := copy(Setup^^.ColorTable[i], LP + 1, length(Setup^^.ColorTable[i]) - LP);
											end;
									end;
								if not FoundIt then
									writelog(stringOf(' Did not find DXF color: ', root^.colornum : 1, ' (defaulted to black)'));
								if not WritelnString(IllFIleNo, stringOf(CurrentIllColor, ' K %=', root^.colornum : 1)) then
									exit(PrintColors);
								if not WritelnString(IllFIleNo, stringOf(CurrentIllColor, ' k  %=', root^.colornum : 1)) then
									exit(PrintColors);
							end; {if colorsubstitution}
						if SetUp^^.Grouping = full then {only then make Color a Group}
							PrintNote(StringOf('  Color: ', root^.colornum : 3));
							
						PrintElems(root^.ElemData);
						
						if SetUp^^.Grouping = full then {only then make Color a Group}
							if not WritelnString(IllFileNo, 'U %colour grouping') then
								exit(PrintColors);
					end;
				root := root^.next;
			end;
	end;

	{---------------------------}
procedure NewLType (var LineType: LTypeTypePtr);
	begin
		if not MemAvail(sizeof(LTypeType)) then 
			Error('[NewLType] ', OutOfMemNr, 0, true,true);
		LineType := LTypeTypePtr(newPtr(sizeof(LTypeType)));
		with LineType^ do
			begin
				LTypeName := '';
				ColorData := nil;
				CurColor := @ColorData;
				next := nil;
			end;
	end;

	{---------------------------}
procedure DisposeLTypes (Root: LTypeTypePtr);
		var
			NextPtr: LTypeTypePtr;
	begin
		while root <> nil do
			begin
				DisposeColors(root^.ColorData);
				NextPtr := root^.next;
				disposePtr(Ptr(root));
				root := NextPtr;
			end;
	end;

	{---------------------------}
function LtypeHasData (LtypePtr: LtypeTypePtr): boolean;
		var
			colorptr: colortypeptr;
	begin
		LtypeHasData := false;
		if LtypePtr = nil then
			begin
			Error('[LtypeHasData] unexpected LtypePtr=nil ', 0, 0, true,true);
			exitToshell;
			end;
		colorptr := ltypePtr^.Colordata;
		while colorptr <> nil do
			begin
				if ColorHasData(ColorPtr) then
					begin
						LtypeHasData := True;
						exit(LtypeHasData);
					end;
				ColorPtr := ColorPtr^.next;
			end;
	end;

	{---------------------------}
procedure PrintLTypes (Root: LTypeTypePtr);
	begin
		while root <> nil do
			begin
				if (Root^.ColorData <> nil) then
					begin
						if SetUp^^.Grouping = full then {only then make LType a Group}
							if not WritelnString(IllFileNo, 'u %linteype grouping') then
								Exit(PrintLTypes);
						if root^.LtypeName = 'BYLAYER' then
							begin
								root^.LtypeName := Curlayer^^.LTypedata^.LTypeName;
							end;
						if not WritelnString(IllFileNo, GetDashString(root^.LtypeName)) then
							exit(PrintLtypes);
						if SetUp^^.Grouping = full then {only then make LType a Group}
							PrintNote(stringOf(' LineType: ', root^.LTypeName));
							
						PrintColors(root^.ColorData);
						
						if SetUp^^.Grouping = full then {only then make LType a Group}
							if not WritelnString(IllFileNo, 'U %linteype grouping') then
								Exit(PrintLTypes);
					end;
				root := root^.next;
			end;
	end;

	{---------------------------}
procedure NewText (var TextData: TextTypePtr);

	begin
		if not MemAvail(sizeof(TextType)) then 
			Error('[NewText] ', OutOfMemNr, 0, true,true);
		TextData := TextTypePtr(newPtr(sizeof(TextType)));
		with TextData^ do
			begin
				TextData := '';
				matrixstring := '';
				next := nil;
			end;
		end;

	{---------------------------}
procedure DisposeTexts (root: TextTypePtr);
		var
			NextPtr: TextTypePtr;
	begin
		while root <> nil do
			begin
				NextPtr := root^.next;
				disposePtr(Ptr(root));
				root := NextPtr;
			end;
	end;

	{---------------------------}
procedure PrintTexts (root: TextTypePtr);
		const
			backslash = '\'; {is special character:when found, change to \\}
		var
			prevalignment: integer;
	begin
		prevalignment := -1; { niet bestaande waarde }
		while root <> nil do
			begin
				with root^ do
					begin
						if alignment <> prevalignment then { alleen schrijven als alignment verandert }
							begin
								if not WriteLnString(IllFileNo, stringof('/_', CurrentIllustratorFont, ' 12 7.2 0 ', alignment : 1, ' z ')) then
									exit(PrintTexts);
								prevalignment := alignment;
							end;
						if not WriteLnString(IllFileNo, matrixstring) then
							exit(PrintTexts);
						if (pos(backslash, Textdata) <> 0) then {backslash in text}
							if (TextData[pos(backslash, Textdata) + 1] <> backslash) then {it's a single one}
								begin
									insert(backslash, TextData, pos(backslash, Textdata)); {replace it with '\\'}
								end;
						if not WritelnString(IllFileNo, textdata) then
							exit(PrintTexts);

						if not writelnString(IllFileNo, 'T') then
							exit(Printtexts);
					end;
					
				{}numElems := numElems +1; {to keep count of progress}
				if numElems/30 = roundtol(numElems/30) then
						DoBalk(numElems);
					
				root := root^.next;
			end;
	end;

	{---------------------------}
procedure NewFontList (var Font: FontListPtr);
	begin
		if not MemAvail(sizeof(FontList)) then 
			Error('[NewFontList] ', OutOfMemNr, 0, true,true);
		font := FontListPtr(newPtr(sizeof(FontList)));
		with font^ do
			begin
				fontname := '';
				next := nil;
				TextData := nil;
				CurText := @TextData;
			end;
	end;

	{---------------------------}
procedure PrintFonts (var Fonts: FontlistPtr);
		var
			Zoekptr: ^FontListPtr;
			i, LP: byte;
			FoundIt: boolean;
	begin
		Zoekptr := @Fonts;
		while Zoekptr^ <> nil do
			begin
				if Zoekptr^^.TextData <> nil then
					begin
						if SetUp^^.Grouping = full then {only then make Fonts a Group}
							if not WritelnString(IllFileNo, 'u %font grouping') then
								exit(PrintFonts);
						if SetUp^^.Grouping = full then {only then make Fonts a Group}
							PrintNote(Stringof('   TextFont: ', Zoekptr^^.FontName));
						if SetUp^^.FontSubstitution then
							begin
								CurrentIllustratorFont := 'Courier'; {if its not in DXF list}
								FoundIt := false;
								for i := 1 to maxNumFonts do
									begin
										LP := pos('\', Setup^^.FontTable[i]);
										if copy(Setup^^.FontTable[i], 1, LP - 1) = Zoekptr^^.FontName then
											begin
												foundIt := true;
												CurrentIllustratorFont := copy(Setup^^.FontTable[i], LP + 1, length(Setup^^.FontTable[i]) - LP);
											end;
									end;
								if not FoundIt then
									writelog(stringOf(' Didnt find font: ', Zoekptr^^.FontName, ' (defaulted to Courier)'));
							end
						else {no font substitution, always Courier}
							CurrentIllustratorFont := 'Courier';
						PrintTexts(Zoekptr^^.TextData);
						if SetUp^^.Grouping = full then {only then make Fonts a Group}
							if not WritelnString(IllFileNo, 'U %font grouping') then
								exit(PrintFonts);
					end;
				Zoekptr := @Zoekptr^^.next;
			end;
	end;

	{---------------------------}
procedure DisposeFontList (var Font: FontlistPtr);
		var
			Zoekptr: ^FontListPtr;
	begin
		Zoekptr := @Font;
		while Zoekptr^ <> nil do
			begin
				DisposeTexts(Zoekptr^^.Textdata);
				Zoekptr := @Zoekptr^^.next;
			end;
	end;

	{---------------------------}
procedure NewLayer (var Layer: LayerTypePtr);
	begin
		if not MemAvail(sizeof(LayerType)) then 
			Error('[NewLayer] ', OutOfMemNr, 0, true,true);
		Layer := LayerTypePtr(newPtr(sizeOf(LayerType)));
		with Layer^ do
			begin
				LayerName := '';
				Fonts := nil;
				LTypeData := nil;
				CurLtype := @LTypeData;
				next := nil;
			end;
	end;

	{---------------------------}
procedure DisPoseLayers;
		var
			zoekPtr: LayerTypePtr;
			NextPtr: LayerTypePtr;
	begin
		
		ZoekPtr := root;
		while ZoekPtr <> nil do
			begin
				DisPoseLTypes(ZoekPtr^.LtypeData);
				DisPoseFontList(ZoekPtr^.fonts);
				NextPtr := ZoekPtr^.next;
				
				ZoekPtr := NextPtr;
				disposePtr(Ptr(ZoekPtr));
			end;
		root := nil;
		
	end;

	{---------------------------}
function LayerHasData (LayerPtr: LayerTypePtr): boolean;
		var
			LtypePtr: LtypeTypePtr;
	begin
		LayerHasData := False;
		if LayerPtr^.fonts <> nil then
			begin
				LayerHasData := True;
				exit(LayerHasData);
			end;
		LtypePtr := LayerPtr^.LTypeData;
		while LtypePtr <> nil do
			begin
				if LtypeHasData(LtypePtr) then
					begin
						LayerHasData := True;
						Exit(LayerHasData);
					end;
				LtypePtr := LtypePtr^.next;
			end;
	end;

	{---------------------------}
function layercount: integer;
		var
			Zoekptr: layerTypePtr;
			result: integer;
	begin
		Zoekptr := root;
		result := 0;
		while zoekptr <> nil do
			begin
				result := result + 1;
				zoekptr := zoekptr^.next;
			end;
		layercount := result;
	end;

	{---------------------------}
procedure PrintLayers (maxx, maxy: real);
		var
			ZoekPtr: LayerTypePtr;
	begin
		texty := maxy;
		textx := maxx;
		ZoekPtr := root;
		while ZoekPtr <> nil do
			begin
				if LayerHasData(ZoekPtr) then
					begin
						if (ZoekPtr^.LtypeData <> nil) or (ZoekPtr^.fonts <> nil) then
							begin
								if not (SetUp^^.Grouping = none) then {Group by layer or full}
									if not WritelnString(IllFileNo, 'u %layer grouping') then
										exit(PrintLayers);
								if (SetUp^^.TargetAppl = Ill50) then {Illlustrator 5.0 layers}
									begin
										if not WritelnString(IllFileNo, '%A15_BeginLayer') then
											exit(PrintLayers);
										if not WritelnString(IllFileNo, stringOf('1 1 1 1 0 0 ', LayerColor : 1, ' 0 0 0 Lb')) then
											exit(PrintLayers);
										LayerColor := LayerColor + 1;
										if LayerColor > 8 then
											LayerColor := 0;
										if not WritelnString(IllFileNo, concat('(', ZoekPtr^.layername, ') Ln')) then
											exit(PrintLayers);
									end;
								if not (SetUp^^.Grouping = none) then {Group by layer or full}
									PrintNote(stringOf('Layer: ', ZoekPtr^.layername));
									
								PrintLTypes(ZoekPtr^.LtypeData);
								PrintFonts(ZoekPtr^.fonts);
								
								if not (SetUp^^.Grouping = none) then {Group by layer or full}
									if not WritelnString(IllFileNo, 'U %layer grouping') then
										exit(PrintLayers);
								if (SetUp^^.TargetAppl = Ill50) then {Illlustrator 5.0 layers}
									begin
										if not WritelnString(IllFileNo, 'LB') then
											exit(PrintLayers);
										if not WritelnString(IllFileNo, '%A15_EndLayer--') then
											exit(PrintLayers);
									end;
							end;
					end;
				ZoekPtr := ZoekPtr^.next;
			end;
	end;

	{---------------------------}
procedure SetNewLayer (LayerName: NameString);
		var
			ZoekLayer: LayerHandle;
	begin
		if CurLayer^ <> nil then
			if CurLayer^^.LayerName = LayerName then
				exit(SetNewLayer);
		ZoekLayer := @Root;
		repeat
			if ZoekLayer^ = nil then
				begin
					NewLayer(Zoeklayer^);
					ZoekLayer^^.LayerName := LayerName;
				end
			else if ZoekLayer^^.LayerName = 'Noname' then
				ZoekLayer^^.LayerName := LayerName
			else if ZoekLayer^^.LayerName <> LayerName then
				ZoekLayer := @ZoekLayer^^.next;
		until (ZoekLayer^ <> nil) and (ZoekLayer^^.LayerName = LayerName);
		CurLayer := ZoekLayer;
	end;

	{---------------------------}
procedure SetNewLType (LTypeName: NameString);
		var
			ZoekLType: LTypeHandle;
	begin
		if CurLayer^ = nil then
			SetNewLayer('Noname');
		if CurLayer^^.CurLType^ <> nil then
			if Curlayer^^.CurLType^^.LTypeName = LTypeName then
				exit(SetNewLtype);
		ZoekLType := @CurLayer^^.LTypeData;
		repeat
			if ZoekLType^ = nil then
				begin
					NewLType(ZoekLType^);
					ZoekLType^^.LTypeName := LTypeName;
				end
			else if ZoekLType^^.LTypeName = 'Noname' then
				ZoekLType^^.LTypeName := LTypeName
			else if ZoekLType^^.LTypeName <> LTypeName then
				ZoekLType := @ZoekLType^^.next;
		until (ZoekLtype^ <> nil) and (ZoekLtype^^.LTypeName = LTypeName);
		CurLayer^^.CurLtype := ZoekLType;
	end;

	{---------------------------}
procedure SetNewColor (ColorNum: Integer);
		var
			ZoekColor: ColorHandle;
	begin
		if CurLayer^ = nil then
			SetNewLayer('Layer: Noname');
		if CurLayer^^.CurLType^ = nil then
			SetNewLtype('Noname');
		if CurLayer^^.CurLType^^.CurColor^ <> nil then
			if CurLayer^^.CurLtype^^.CurColor^^.ColorNum = Colornum then
				exit(SetNewColor);
		ZoekColor := @CurLayer^^.CurLType^^.ColorData;
		repeat
			if ZoekColor^ = nil then
				begin
					NewColor(ZoekColor^);
					ZoekColor^^.ColorNum := ColorNum;
				end
			else if ZoekColor^^.ColorNum = -1 then
				ZoekColor^^.ColorNum := ColorNum
			else if ZoekColor^^.ColorNum <> ColorNum then
				ZoekColor := @ZoekColor^^.next;
		until (ZoekColor^ <> nil) and (ZoekColor^^.ColorNum = ColorNum);
		CurLayer^^.CurLtype^^.CurColor := ZoekColor;
	end;

	{---------------------------}
procedure AddElem (Etype: char; p1, p2, p3: punttype);
	begin
		if CurLayer^ = nil then
			SetNewLayer('Layer: Noname');
		if CurLayer^^.CurLType^ = nil then
			SetNewLtype('Noname');
		if CurLayer^^.CurLtype^^.CurColor^ = nil then
			SetNewColor(-1);
		if EType = 'w' then
			begin
				p1.x := p1.x * illscale;
			end
		else
			begin
				transform(IllCtm, p1.x, p1.y);
				transform(IllCtm, p2.x, p2.y);
				transform(IllCtm, p3.x, p3.y);
			end;
		with CurLayer^^.CurLtype^^.CurColor^^ do
			begin
				NewElemtype(CurElem^);
				if CurElem^ = nil then
					begin
					Error('[AddElem] ', OutOfMemNr, 0, true,true);
					exitToshell;
					end;
				with CurElem^^ do
					begin
						ElemName := Etype;
						x1 := p1.x;
						x2 := p2.x;
						x3 := p3.x;
						y1 := p1.y;
						y2 := p2.y;
						y3 := p3.y;
					end;
				CurElem := @CurElem^^.next;
			end;
		{}numElems := NumElems+1;
	end;

	{---------------------------}
procedure AddTekst (var theText: TextRecord);
		var
			TxtCtm: Matrix;
			mtxstring: string;
			width: real;
			align: integer;
			FontHandle: ^FontListPtr;
	begin
		width := 0.6 * theText.height * length(theText.tekst) * theText.scale;
		if width = 0 then
			exit(AddTekst);
		if CurLayer^ = nil then
			SetNewLayer('Layer: Noname');
		with TheText do
			begin
				TxtCtm := IllCtm;
				case justtype of
					0: 
						begin {text is left justified }
							Translate(txtCtm, x, y);
							align := 0;
						end;
					1: 
						begin {text is centred along its baseline}
							translate(TxtCtm, alx, aly);
							align := 1;
						end;
					2: 
						begin {text is right justified}
							translate(TxtCtm, alx, aly);
							align := 2;
						end;
					3: 
						begin {text is aligned between two points (height varies)}
							translate(TxtCtm, (x + alx) / 2, (y + aly) / 2);
							align := 1;
							height := sqrt(sqr(x - alx) + sqr(y - aly)) / (length(tekst) * 0.6);
						end;
					4: 
						begin {text is "middle" (fully) centred}
							translate(TxtCtm, alx, aly - height / 2);
							align := 1;
						end;
					5: 
						begin{text is fit between two points (width varies)}
							translate(TxtCtm, (x + alx) / 2, (y + aly) / 2);
							align := 1;
							scalectm(TxtCtm, sqrt(sqr(x - alx) + sqr(y - aly)) / width, 1);
						end;
					otherwise
						begin {unknown alignment, justify left }
							Translate(txtCtm, x, y);
							align := 0;
						end;
				end; {case}
				rotate(txtCtm, angle);
				scalectm(txtCtm, scale, 1);
				if bitand(textgenflags, 2) = 2 then
					scalectm(txtctm, -1, 1);  { spiegelbeeld }
				if bitand(textgenflags, 4) = 4 then
					scalectm(txtctm, 1, -1); {ondersteboven}
				scalectm(txtctm, height / 12, height / 12);
				mtxstring := stringof('[', txtctm[1, 1] : 4 : 4, ' ', txtctm[1, 2] : 4 : 4, ' ');
				mtxstring := concat(mtxstring, stringof(txtctm[2, 1] : 4 : 4, ' ', txtctm[2, 2] : 4 : 4, ' '));
				mtxstring := concat(mtxstring, stringof(txtctm[3, 1] : 4 : 4, ' ', txtctm[3, 2] : 4 : 4, ']e'));
				tekst := stringof('(', tekst, ')t');
			end;
		with CurLayer^^ do
			begin
				FontHandle := @fonts;
				repeat
					if FontHandle^ = nil then
						begin
							NewFontList(FontHandle^);
							fontHandle^^.fontname := TheText.Font;
						end
					else if FontHandle^^.fontname <> TheText.Font then
						FontHandle := @FontHandle^^.next;
				until FontHandle^^.fontname = TheText.Font;
				with FontHandle^^ do
					begin
						NewText(CurText^);
						with Curtext^^ do
							begin
								alignment := align;
								TextData := TheText.Tekst;
								matrixstring := mtxstring;
								FontName := TheText.Font;
							end;
						CurText := @CurText^^.next;
					end;
			end;
		{}numElems := NumElems+1;
	end;

	{---------------------------}
procedure InitLayers;
		var
			tel: integer;
	begin
		NumElems := 0;
		layerColor := 0;
		ItsABlock := false; {}
		Root := nil;
		CurLayer := @Root;
		for tel := 1 to maxltypes do
			begin
				LtypeDash[tel].LTypeName := '';
				LtypeDash[tel].Dashstring := '';
			end;
	end;

end.