unit Sections;

{ ############  DXF TO ILLUSTRATOR #############################}
{    1991-1994 Universiteit Utrecht, FRW, Vakgroep Kartografie &                            }
{                           International Institute for Aerospace Survey And Earth Sciences ITC  }
{ 	Version 1.1.2  -  Main Parts Programmed by J.C. Blankert , 1991                            }
{ 	Version 2.0     -  Changes programmed by B.J. Kbben, 1992                 		            }
{ 	 							- Scaling: IllMaxX and Y are now functions  instead of constants    }
{ 	 											These are set in the new unit Scale					            }
{							- ReadStyleTable to get font List, nov. 93											}
{							- ReadStyleTable changed to new fontLyst structure, march 94			}
{		Version 2.1 -  ReadStyleTable & now writes to tempTable - 8/94							}
{							- ReadStyleTable to get color List - 9/94						
		Version 2.2OS - code and app made OpenSource July 2002 BJK					}
{ ########################################################}

interface
	uses
		{UIP}
	QuickDraw,memory,windows,fp,QuickDrawText,fonts,menus,processes,
	Dialogs,Types,TextEdit,TextUtils,toolutils,OSutils,
		{MyUnits}
			globals, ErrorsAndBeeps, log, Scale, myFiles, myWindows,  setup, 
			Ctm, IllFile, Layers, Collect, Entities,tables;
	
	var
		EndTime: longint; { systeem datum aan einde conversie}


	function ReadSections: boolean;
	function readsection (why: reason) : boolean;

implementation

	var
		group: grouptype;
		maxx, maxy, minx, miny, temp: extended;
		CurrentSection : str255;

	type
		listptr = ^list;
		list = record
				group: grouptype;
				next: listptr;
			end;


	

{-----------------------------}
	procedure findlimits; { bepaal afmetingen dxf tekening }
		var
			CurrentPos: Longint;
			CurrentGroup: Grouptype;
	begin
		CurrentGroup.strdat := nil;
		CopyGroup(Group, CurrentGroup);
		CurrentPos := GetFilePos;
		minx := 1e25;
		miny := minx;
		maxx := -1e25;
		maxy := maxx;
		with group do
			while (strdat^ <> 'EOF') and readgroup(group, normal) do
				begin
				WaitingForStr := 'EOF';
				checkAbort;
				if aborted then exit(findlimits);
					case code of
						10..18: 
							if minx > realdat then
								minx := realdat
							else if maxx < realdat then
								maxx := realdat;
						20..28: 
							if miny > realdat then
								miny := realdat
							else if maxy < realdat then
								maxy := realdat;
						otherwise
							;
					end;{case}
				end;{while}
		
		SetFilePos(CurrentPos);
		CopyGroup(CurrentGroup, group);
		if CurrentGroup.datatype = STRINGTP then
				disposePtr(Ptr(CurrentGroup.strdat));
		if (maxx - minx) > (maxy - miny) then
			scrscale := ScrGetMaxX / (maxx - minx)
		else
			scrscale := ScrGetMaxY / (maxy - miny);
		if SetUp^^.scaling = toCustom then
			begin
				illscale := IllMaxX
			end
		else
			begin
				if IllMaxX / (maxx - minx) < IllMaxY / (maxy - miny) then
					illscale := IllMaxX / (maxx - minx)
				else
					illscale := IllMaxY / (maxy - miny);
			end;
		with Limits do
			begin
				Xmin := minx;
				Ymin := miny;
				Xmax := maxx;
				Ymax := maxy;
				IllXmin := 0;
				IllYmin := 0;
				IllXmax := ((maxx * illScale) / points) - ((minx * illScale) / points);
				IllYmax := ((maxy * illScale) / points) - ((miny * illScale) / points);
			end;
		bbox_set := TRUE;
		
	end;

{--------------------------------------------------------------------------------------}
	procedure findcolors; { find colors in entities }
		var
			CurrentPos: Longint;
			CurrentGroup: Grouptype;
			Colstring: string;
			ColListed, itsFull: boolean;
			i: byte;

	begin
		CurrentGroup.strdat := nil;
		CopyGroup(Group, CurrentGroup);
		CurrentPos := GetFilePos;
		itsFull := false;
		with group do
			while (strdat^ <> 'EOF') and not EOFencountered and readgroup(group, getColors) and not itsFull do
				begin
				WaitingForStr := 'Group Code 62';
				if (group.code = 62) then
					begin
						ColListed := false;
						ColString := stringOf(group.intdat : 1);
						for i := 1 to maxNumFonts do
							if Colstring = copy(TempTable[i], 1, pos('\', tempTable[i]) - 1) then
								ColListed := true;
						if not Collisted then
							begin
								i := 0;
								repeat
									i := i + 1;
									if i > maxNumFonts then
										begin
											error(stringList(ToManyFontsNr), 0, 0, false,false);
											itsFull := true;
										end;
								until length(TempTable[i]) <= 1;
								TempTable[i] := concat(Colstring, '\');
								numTableThings := numTableThings + 1;
								play(ping);
							end; {not Collisted}
					end; {group=62}
				end; {while...}
		SetFilePos(CurrentPos);
		CopyGroup(CurrentGroup, group);
	end;
	
{-----------------------------}
	function readheader(why: reason): boolean;
		var
			EXTFound: boolean;
	begin
		ReadHeader := false;
		Bbox_Set := False;
		EXTfound := False;
		if not readgroup(group, normal) then
			exit(readheader);
		with group do
			while not ((datatype = STRINGTP) and (strdat^ = 'ENDSEC')) do
				begin
					WaitingForStr := 'ENDSEC';
					if (datatype = STRINGTP) and (strdat^ = '$ACADVER') then
						begin
							repeat
								if not (readgroup(group, normal)) then
									exit(readheader);		
								if group.code = 1 then
									begin
									ACADversion := group.strdat^;
									if ACADversion = 'AC1006' then ACADversion := 'R10'
									else if ACADversion ='AC1009' then ACADversion := 'R11/R12'
									else if ACADversion ='AC1011' then ACADversion := 'R13'
									else ACADversion := concat(stringList(unknownNr), '(',ACADVersion,')');					
									end;
							until (group.code = 9) or (group.code = 0);
						end
					else if (datatype = STRINGTP) and (strdat^ = '$EXTMAX') then
						begin
							EXTFound := True;
							Bbox_Set := True;
							repeat
								if not (readgroup(group, normal)) then
									exit(readheader);
								if why = normal then
									doBalk(GetfilePos);			
								if group.code = 10 then
									maxx := group.realdat
								else if group.code = 20 then
									maxy := group.realdat;
							until (group.code = 9) or (group.code = 0);
						end
					else if (datatype = STRINGTP) and (strdat^ = '$EXTMIN') then
						begin
							EXTFound := True;
							BBox_Set := True;
							repeat
								if not (readgroup(group, normal)) then
									exit(readheader);
								if why = normal then
									doBalk(GetfilePos);
								if group.code = 10 then
									minx := group.realdat
								else if group.code = 20 then
									miny := group.realdat;
							until (group.code = 9) or (group.code = 0);
						end
					else if (datatype = STRINGTP) and (strdat^ = '$LIMMIN') then
						begin
							BBox_Set := True;
							if not EXTFound then
								repeat
									if not (readgroup(group, normal)) then
										exit(readheader);
									if why = normal then
										doBalk(GetfilePos);
									if group.code = 10 then
										minx := group.realdat
									else if group.code = 20 then
										miny := group.realdat;
								until (group.code = 0) or (group.code = 9)
							else if not readgroup(group, normal) then
								exit(readheader);
						end
					else if (datatype = STRINGTP) and (strdat^ = '$LIMMAX') then
						begin
							BBox_Set := True;
							if not EXTFound then
								repeat
									if not (readgroup(group, normal)) then
										exit(readheader);
									if why = normal then
										doBalk(GetfilePos);
									if group.code = 10 then
										maxx := group.realdat;
									if group.code = 20 then
										maxy := group.realdat;
								until (group.code = 9) or (group.code = 0)
							else if not readgroup(group, normal) then
								exit(readheader);
						end
					else if not (readgroup(group, normal)) then
						exit(readheader);
				end;
		if not BBox_Set then
			begin
				ReadHeader := True;
				exit(readheader);
			end;
		if ScrGetMaxX / (maxx - minx) < ScrGetMaxY / (maxy - miny) then
			scrscale := ScrGetMaxX / (maxx - minx)
		else
			scrscale := ScrGetMaxY / (maxy - miny);
		if SetUp^^.scaling = toCustom then
			begin
				illscale := IllMaxX
			end
		else
			begin
				if IllMaxX / (maxx - minx) < IllMaxY / (maxy - miny) then
					illscale := IllMaxX / (maxx - minx)
				else
					illscale := IllMaxY / (maxy - miny);
			end;
		with Limits do
			begin
				Xmin := minx;
				Ymin := miny;
				Xmax := maxx;
				Ymax := maxy;
				IllXmin := 0;
				IllYmin := 0;
				IllXmax := ((maxx * illScale) / points) - ((minx * illScale) / points);
				IllYmax := ((maxy * illScale) / points) - ((miny * illScale) / points);
			end;
		ReadHeader := True;
	end;

{-----------------------------------------}
	function ReadLayerTable (why: reason): boolean;
		var
			Colstring: string;
			ColListed: boolean;
			i: byte;

	begin
		ReadLayerTable := False;
		if why = getColors then
			for i := 1 to maxNumFonts do {clear Table}
				TempTable[i] := '\';
		if not ReadGroup(group, why) then
			exit(ReadLayerTable);
		while (group.strdat^ <> 'ENDTAB') do
			begin
				WaitingForStr := 'ENDTAB';
				if group.code = 2 then
					SetNewLayer(Group.strdat^);
				if (group.code = 62) and (why = getColors) then
					begin
						ColListed := false;
						ColString := stringOf(group.intdat : 1);
						for i := 1 to maxNumFonts do
							if Colstring = copy(TempTable[i], 1, pos('\', tempTable[i]) - 1) then
								ColListed := true;
						if not Collisted then
							begin
								i := 0;
								repeat
									i := i + 1;
									if i > maxNumFonts then
										begin
											error(stringList(ToManyFontsNr), 0, 0, false,false);
											ReadLayerTable := True;
											exit(ReadLayerTable);
										end;
								until length(TempTable[i]) <= 1;
								TempTable[i] := concat(Colstring, '\');
								numTableThings := numTableThings + 1;
								play(ping);
							end; {not Collisted}
					end; {group=62; why=getColors}
				if not readgroup(group, why) then
					exit(ReadLayerTable);
				if why = normal then
					doBalk(GetfilePos);
			end;
		ReadLayerTable := True;
	end;

{-----------------------------------------}
	function ReadStyleTable (why: reason): boolean;
		var
			stylestring: string;
			fontListed: boolean;
			i: byte;

	begin
		ReadStyleTable := False;
		if why = getFonts then
			for i := 1 to maxNumFonts do {clear Table}
				TempTable[i] := '\';
		if not ReadGroup(group, normal) then
			exit(ReadStyleTable);
		while (group.strdat^ <> 'ENDTAB') do
			begin
				WaitingForStr := 'ENDTAB';
				if group.code = 2 then
					begin
						if why = getFonts then
							begin
								stylestring := Group.strdat^;
								if not ReadGroup(group, normal) then
									exit(ReadStyleTable);
								if (group.code = 70) and (group.intdat <> 65) then {its not a shape file load request}
									begin
										fontListed := false;
										for i := 1 to maxNumFonts do
											if stylestring = copy(TempTable[i], 1, pos('\', TempTable[i]) - 1) then
												fontListed := true;
										if not fontlisted then
											begin
												i := 0;
												repeat
													i := i + 1;
													if i > maxNumFonts then
														begin
															error(stringList(ToManyFontsNr), 0, 0, false,false);
															ReadStyleTable := True;
															exit(ReadStyleTable);
														end;
												until length(TempTable[i]) <= 1;
												TempTable[i] := concat(stylestring, '\');
												numTableThings := numTableThings + 1;
												play(ping);
											end;
									end;
							end; {if why =getfonts}
					end;
				if not readgroup(group, normal) then
					exit(ReadStyleTable);
				if why = normal then
					doBalk(GetfilePos);
			end;
		ReadStyleTable := True;
	end;

{-----------------------------------------}

	function ReadLTypeTable(why: reason): boolean;
		var
			regel, dashstring: string;
			code, tel, dashcount, dashnum: integer;
			dashlen: real;
	begin { vult in unit layers gedefineerde array LtypeDash }
		ReadLtypeTable := False;
		tel := 0;
		dashcount := 0;
		if not ReadLine(DxfFileNo, regel) then
			exit(ReadLTypeTable);
		ReadString(regel, code);
		if not ReadLine(DxfFileNo, regel) then
			exit(ReadLTypeTable);
		while (regel <> 'ENDTAB') do
			begin
				WaitingForStr := 'ENDTAB';
				if tel <= maxltypes then
					begin
						case code of
							2: 
								begin
									tel := tel + 1;
									LTypeDash[tel].LtypeName := regel;
									dashstring := '';
								end;
							73: 
								begin
									readstring(regel, DashNum);
									dashcount := 0;
									if DashNum = 0 then
										LtypeDash[tel].DashString := '[]0 d';
								end;
							49: 
								begin
									dashcount := dashcount + 1;
									if (dashcount <= dashnum) then
										begin
											readstring(regel, dashlen);
											regel := stringof(' ', abs(dashlen * 72 / 2.54) : 3 : 2);
											dashstring := concat(dashstring, regel);
										end;
									if dashcount = dashnum then
										begin
											if dashstring <> '' then
												begin
													delete(dashstring, 1, 1);
													dashstring := concat('[', dashstring);
													dashstring := concat(dashstring, ']0 d');
												end
											else
												dashstring := '[]0 d';
											LtypeDash[tel].dashstring := dashstring;
										end;
								end;
							otherwise
								;
						end; {case}
					end;
				if not ReadLine(DxfFileNo, regel) then
					exit(ReadLTypeTable);
				ReadString(regel, code);
				if not ReadLine(DxfFileNo, regel) then
					exit(ReadLTypeTable);
			end;
		ReadLtypeTable := True;
	end;

	{-----------------------------}
	function ReadTables (why: reason): boolean;
	
	begin
		ReadTables := False;
		if not readgroup(group, normal) then
			exit(ReadTables);
		while group.strdat^ <> 'ENDSEC' do
			begin
				WaitingForStr := 'ENDSEC';
				if not readgroup(group, normal) then
					exit(ReadTables);
				if why = normal then
					doBalk(GetfilePos);			
				if (group.code = 2) then
					begin
						if (group.strdat^ = 'LAYER') then
							begin
								if not ReadLayerTable(why) then
									exit(ReadTables);
							end
						else if (group.strdat^ = 'LTYPE') then
							begin
								if not ReadLTypeTable(why) then
									exit(ReadTables);
							end
						else if (group.strdat^ = 'STYLE') then
							begin
								if not ReadStyleTable(why) then
									exit(ReadTables);
							end;
					end;
			end;
		ReadTables := True;
	end;
	
{-----------------------------}
	function ReadObjects (why: reason): boolean;
	
	begin
		ReadObjects := False;
		if not readgroup(group, normal) then
			exit(ReadObjects);
		while group.strdat^ <> 'ENDSEC' do
			begin
				WaitingForStr := 'ENDSEC';
				if not readgroup(group, normal) then
					exit(ReadObjects);		
			end;
		ReadObjects := True;
	end;
	
	{-----------------------------}
	function ReadClasses (why: reason): boolean;
	
	begin
		ReadClasses := False;
		if not readgroup(group, normal) then
			exit(ReadClasses);
		while group.strdat^ <> 'ENDSEC' do
			begin
				WaitingForStr := 'ENDSEC';
				if not readgroup(group, normal) then
					exit(ReadClasses);
			end;
		ReadClasses := True;
	end;
	
	{-----------------------------}
	function ReadUnknown (why: reason): boolean;
	
	begin
		ReadUnknown := False;
		if not readgroup(group, normal) then
			exit(ReadUnknown);
		while group.strdat^ <> 'ENDSEC' do
			begin
				WaitingForStr := 'ENDSEC';
				if not readgroup(group, normal) then
					exit(ReadUnknown);		
			end;
		ReadUnknown := True;
	end;
	
	
		{-----------------------------}
	function doReadEntities (why: reason): boolean;
	
		var
			scrmaxx, scrmaxy, scrminx, scrminy: real;
			
			
			function flushEntities(why:reason):boolean;
			
				begin
				flushEntities := false;
				if not readgroup(group, normal) then
					exit(flushEntities);
				while group.strdat^ <> 'ENDSEC' do
					begin
					doBalk(GetfilePos);
					WaitingForStr := 'ENDSEC';
					if not readgroup(group, normal) then
						exit(flushEntities);
					end;
				flushEntities := true;
				end;
				
				
	begin
		
		doReadEntities := False;
		if why = getColors then
			begin
				findColors;
			end;
		if (why <> normal) then {don't have to do other things}
			begin
			if flushEntities(why) then
				doReadEntities := true;
			exit(doReadEntities);
			end;
		if not bbox_set then
			begin
				if infoOnly then
						SetInfoMessage(ExtentsNr)
				else
						SetMessage(ExtentsNr, true); 
				findlimits;
				if not bbox_set then
					begin
					if infoOnly then
						Error('', NoLimitsNr, 0, false,false)
					else
						Error('', NoLimitsNr, 0, true,false);
					exit(doReadEntities);
					end;
			end;
			
		if infoOnly then {all's done, stop here}
			begin			
			SetMessage(0, false); {to force last message to be erased}
			if flushEntities(why) then
				doReadEntities := true;
			exit(doReadEntities);
			end;
			
		if aborted then exit(doReadEntities);
		InitCtm(IllCtm);
		translate(IllCtm, 1, 1);
		ScaleCtm(IllCtm, IllScale, IllScale);
		translate(IllCtm, -minx, -miny);
		if why=normal then 
			if not SendPsHeader(minx, miny, maxx, maxy) then
				exit(doReadEntities);
		
		InitCtm(ScrCtm);
		translate(ScrCtm, drawingRect.left, drawingRect.top);
		scaleCtm(ScrCtm, ScrScale, -ScrScale);
		scrminx := minx ;
		scrminy := miny ;
		scrmaxx := maxx ;
		scrmaxy := maxy ;
		translate(ScrCtm, -scrminx, -scrmaxy);
		transform(ScrCtm, scrminx, scrminy);
		transform(ScrCtm, scrmaxx, scrmaxy);
		
		if not doBackground then
			begin 
			setRect(drawingRect,roundtol(scrminx),roundtol(scrmaxy),roundtol(scrmaxx),roundtol(scrminy));
			insetRect(drawingRect,-2,-2);
			BoxOutLine(DrawingRect,2);
			end;
		
		SetMessage(RdEntitiesNr, true); {Reading Entities}
		
		if not ReadEntities(normal) then {in unit Entities}
			Error(concat(stringList(UnXElementNr),'ENTITIES'), 0, DXFlineNr, true,false);
									
		if why=normal then
			DoBalk(BalkMax);
		
		doReadEntities := true;	
		
	end;
	

						
	{-----------------------------}
	function readblocks(why: reason): boolean;
		
	var
			hatch_asked,BlockDefEnded: boolean;
	begin
		readblocks := False;
		hatch_asked := False;
		blockcount := 0;
		BlockDefEnded := false;
		HLock(Handle(BlockList));
		repeat
			if not readgroup(group, normal) then
				begin
				HUnLock(Handle(BlockList));
				exit(readblocks);
				end;
			if why = normal then
				doBalk(GetfilePos);
			if (group.datatype = STRINGTP) and (group.strdat^ = 'BLOCK') then
				begin
				blockcount := blockcount + 1;
				BlockDefEnded := false;
				if (blockcount > MAXBLOCKS) then
					begin
					error(stringOf(stringlist(BlockOFlwNr), ' (', MAXBLOCKS : 1, ').'), 0, 0, true,false);
					HUnLock(Handle(BlockList));
					exit(readblocks);
					end;
				BlockList^^[BlockCount].BlockType := 0;
				BlockList^^[BlockCount].Layername := '';
				
				repeat
				
					if not readgroup(group, normal) then
						begin
						HUnLock(Handle(BlockList));
						exit(readblocks);
						end;
					if group.code = 0 then
						BlockDefEnded := true;
					if not BlockDefEnded then
						begin 
							case group.code of
								1,3,5,1000..1071:
									; { ignore}
								100:
									; {Subclass Marker => ignore}
								2: 
									BlockList^^[BlockCount].BlockName := group.strdat^;
								8: 
									BlockList^^[BlockCount].Layername := group.strdat^;
								10: 
									BlockList^^[BlockCount].BlockBaseX := group.realdat;
								20: 
									BlockList^^[BlockCount].BlockBaseY := group.realdat;
								30: 
									BlockList^^[BlockCount].BlockBaseZ := group.realdat;
								70: 
									begin
										BlockList^^[BlockCount].BlockType := group.intdat;
										if (not hatch_asked) and (bitand(group.intdat, 1) = 1) then
											begin
												BlockHatch := AskHatch;
											end;
										hatch_asked := True;
									end;
								otherwise
									; {do nothing}
							end;{case}
						BlockList^^[BlockCount].FilePos := GetFilePos;
						BlockList^^[BlockCount].DXFstartLine := DXFlineNr;
						end;
					doBalk(GetfilePos);
					WaitingForStr := 'ENDBLK';
					
				until (group.datatype = STRINGTP) and (group.strdat^ = 'ENDBLK');
				
			end; {reading this block}
			WaitingForStr := 'ENDSEC';
			
		until (group.datatype = STRINGTP) and (group.strdat^ = 'ENDSEC');
		ReadBlocks := True;
		HUnLock(Handle(BlockList));
	end;


{--------------------------------------------------------------------------------------}
	function readsection (why: reason): boolean;
	
	var
		SectionFinished : boolean;
	
	begin
		readsection := false;
		if aborted then
			exit(readsection);
		
		
		if not readgroup(group, normal) then
			begin
				{}warning(concat('First Readgroup returned false '),DXFLINENR,true);
				exit(readsection);
			end;
			
		with group do
			begin
			
			SectionFinished := false;
			if (strdat^ = 'EOF') or aborted then
			  begin
				readsection := false; {'cos whe're finished}
				{}warning(concat('Found section ','EOF'),DXFLINENR,false);
				SectionFinished := true;
				WaitingForStr := '';
				exit(readsection);
			  end
			else if (strdat^ <> 'SECTION') then
				begin
				Error('SECTION', ExpectedNr, DXFlineNr, true,false);
				exit(readsection);
				end;
			  
				
			while (not SectionFinished) and (not aborted) do
				begin
					if not readgroup(group, normal) then
						exit(readsection);
					if strdat^ = 'HEADER' then
						begin
						CurrentSection := strdat^;
						if aborted then exit(readsection);
						{}warning(concat('Found section ',CurrentSection),DXFLINENR,false);
							if infoOnly then
								SetInfoMessage(RdHeaderNr)
							else
								SetMessage(RdHeaderNr, true); { Reading Header }
							if not readheader(why) then
								begin
									if Bbox_Set then
									Error(concat(stringList(UnXElementNr),'',CurrentSection,'.'), 0, DXFlineNr, true,false);
									exit(readsection);
								end;
						SectionFinished := true;
						WaitingForStr := '';
						end
					else if strdat^ = 'CLASSES' then {R13 section-no graphical data-> ignore}
						begin
							CurrentSection := strdat^;
							if aborted then exit(readsection);
							{}warning(concat(stringList(UnusedSectionNr),'',CurrentSection ,' (',stringList(noGraphicsNr),').'),DXFlineNr,true);															
							if not readClasses(why) then
								begin
									Error(concat(stringList(UnXElementNr),'',CurrentSection,'.'), 0, DXFlineNr, true,false);
									exit(readsection);
								end;
							SectionFinished := true;
							WaitingForStr := '';
						end
					else if strdat^ = 'TABLES' then
						begin
							CurrentSection := strdat^;
							if aborted then exit(readsection);
						{}warning(concat('Found section ',CurrentSection),DXFLINENR,false);
							if infoOnly then
								SetInfoMessage(PrTablesNr)
							else
								SetMessage(PrTablesNr, true); { Processing Tables }
							if not readtables(why) then
								begin
									Error(concat(stringList(UnXElementNr),'',CurrentSection,'.'), 0, DXFlineNr, true,false);
									exit(readsection);
								end;
							SectionFinished := true;
							WaitingForStr := '';
						end
					else if strdat^ = 'BLOCKS' then
						begin
							CurrentSection := strdat^;
							if aborted then exit(readsection);
						{}warning(concat('Found section ',CurrentSection),DXFLINENR,false);
							if infoOnly then
								SetInfoMessage(PrBlocksNr)
							else
								SetMessage(PrBlocksNr, true); { Processing Blocks }
							if not readblocks(why) then
								begin
									Error(concat(stringList(UnXElementNr),'',CurrentSection,'.'), 0, DXFlineNr, true,false);
									exit(readsection);
								end;
							SectionFinished := true;
							WaitingForStr := '';
						end
					else if (strdat^ = 'ENTITIES') then
						begin
							CurrentSection := strdat^;
							if aborted then exit(readsection);		
						{}warning(concat('Found section ',CurrentSection),DXFLINENR,false);													
							if not doReadEntities(why) then
								begin
									Error('', UnXElementNr, 0, true,false);
									exit(readsection);
								end;
							SectionFinished := true;
							WaitingForStr := '';
						end {entities}
					else if strdat^ = 'OBJECTS' then {R13 section-no graphical data-> ignore}
						begin
							CurrentSection := strdat^;
							if aborted then exit(readsection);		
							warning(concat(stringList(UnusedSectionNr),'',CurrentSection ,' (',stringList(noGraphicsNr),').'),DXFlineNr,true);															
							if not readObjects(why) then
								begin
									Error(concat(stringList(UnXElementNr),'',CurrentSection,'.'), 0, DXFlineNr, true,false);
									exit(readsection);
								end;
							SectionFinished := true;
							WaitingForStr := '';
						end
					else if strdat^ = 'EOF' then {Should not be here, but no major disaster if it is}
						begin
							if aborted then exit(readsection);
							CurrentSection := strdat^;
						{}warning(concat('Found section ',CurrentSection),DXFLINENR,false);
							readsection := false; {'cos whe're finished}
							exit(readsection);
							WaitingForStr := '';
						end	
					else {other -unknown- sections}	
						begin
							CurrentSection := strdat^;
							if aborted then exit(readsection);
							warning(concat(stringList(UnusedSectionNr),'',CurrentSection ,' (',stringList(UnknownSectionNr),').'),DXFlineNr,true);
							if not readUnknown(why) then
								begin
									Error(concat(stringList(UnXElementNr),'',CurrentSection,'.'), 0, DXFlineNr, true,false);
									exit(readsection);
								end;
							SectionFinished := true;
							WaitingForStr := '';
						end;
				end;   {while not SectionFinished}
			readsection := TRUE;
			end; { with group }
	end; {readsection}

	{-----------------------------}
	function ReadSections;
			
	begin
		ReadSections := false;
		BlockCount := 0;
		BlockHatch := True;
		WaitingForStr := '';
		
		if aborted then
			exit(readsections);
		if not DrwWinInited then
			begin
			if infoOnly then
		 		CreateDrawingWindow(concat(stringList(Infonr),'',GetFName(DXFFileno),''))
		 	else
				CreateDrawingWindow(concat(stringList(Generalnr),'',GetFName(DXFFileno),''));
			InitBalk(0,FileSize(DxfFileNo));	
			end;
			
		if not MemAvail(sizeof(strtype)) then 
			Error('[ReadSections:1] ', OutOfMemNr, 0, true,true);
		group.strdat := strptr(newPtr(sizeof(strtype)));
		
		if BlockList <> nil then {clean old one up}
			disposeHandle(Handle(BlockList));
		if not MemAvail(sizeof(DynamicBlocklist)) then 
			Error('[DynamicBlockList]', OutOfMemNr, 0, true,true);
		BlockList := BlockListHandle(newHandle(sizeOf(DynamicBlocklist)));
		if BlockList = nil then
			Error('[DynamicBlockList not allocated]', 0, 0, true,true) 
		else
			begin
				bbox_set := false;
				InitLayers;
				if setinput(DxfFileNo) then
					begin
					Reset(DxfFileNo);
					DXFlineNr := 0;
					 	
					if aborted then
						exit(readsections);
					while readsection(normal) do
						begin
						; {keep reading sections until EOF or Error or Aborted}
						end;
					
					SetMessage(WriteIllFileNr, true); {Writing Illustrator file}
					InitBalk(0,numElems);
					numELems := 0; {reset to start counting again in Printelems & PrintTexts}
					PrintLayers(maxx, maxy);
					GetDateTime(EndTime);
					SetMessage(CnvReadyNr, false);
					DoBalk(BalkMax);
					
					ReadSections := True;
					if not SendPsTrailer(minx, miny, maxx, maxy) then
						Error('[SendPsTrailer failed] ',0,0,true,false); 
					end;
				DisposeLayers;
			end;
		if group.strdat <> nil then disposePtr(Ptr(Group.strdat));
		if BlockList <> nil then {clean old one up}
			disposeHandle(Handle(BlockList));
		WaitingForStr := '';
	end;

end.